<?php

namespace Netzmacher\Tile\Controller;

use \Netzmacher\Tile\Domain\Repository\AreaRepository;
use \Netzmacher\Tile\Domain\Repository\CategoryRepository;
use \Netzmacher\Tile\Domain\Repository\CellRepository;
use \Netzmacher\Tile\Domain\Repository\ContentRepository;
use \Netzmacher\Tile\Domain\Repository\CssRepository;
use \Netzmacher\Tile\Domain\Repository\GroupRepository;
use \Netzmacher\Tile\Domain\Repository\PageRepository;
use \Netzmacher\Tile\Domain\Repository\ProductRepository;
use \Netzmacher\Tile\Domain\Repository\TicketRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * Class AbstractController
 * 
 * @version 0.0.6
 * @since 0.0.1

 */
abstract class AbstractController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{

	/**
	 * areaRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\AreaRepository
	 * @inject
	 */
	protected $areaRepository = null;

	/**
	 * categoryRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\CategoryRepository
	 * @inject
	 */
	protected $categoryRepository = null;

	/**
	 * cellRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\CellRepository
	 * @inject
	 */
	protected $cellRepository = null;

	/**
	 * @var ContentObjectRenderer
	 */
	protected $contentObject;

	/**
	 * contentRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\ContentRepository
	 * @inject
	 */
	protected $contentRepository = null;

	/**
	 * cssRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\CssRepository
	 * @inject
	 */
	protected $cssRepository = null;

	/**
	 * groupRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\GroupRepository
	 * @inject
	 */
	protected $groupRepository = null;

	/**
	 * pageRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\pageRepository
	 * @inject
	 */
	protected $pageRepository = null;

	/**
	 * @var \TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager
	 * @inject
	 */
	protected $persistenceManager;

	/**
	 * ticketRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\ProductRepository
	 * @inject
	 */
	protected $productRepository = null;

	/**
	 * ticketRepository
	 * 
	 * @var \Netzmacher\Tile\Domain\Repository\TicketRepository
	 * @inject
	 */
	protected $ticketRepository = null;

	/**
	 * _getFlexformValue():
	 *
	 * @param string method
	 * @param string line
	 * @param string flexform
	 * @param string label of the sheet
	 * @param string label of the field
	 * @return mixed value from flexform
	 * @access protected
	 * @version 0.5.2
	 * @since 0.5.2
	 */
	protected function _getFlexformValue( $method, $line, $xmlFlexform, $sheet, $field )
	{
		$NSprefix = '';
		$reportDocTag = false;

		$arrFlexform = GeneralUtility::xml2array( $xmlFlexform, $NSprefix, $reportDocTag );

		if( !isset( $arrFlexform[ 'data' ][ $sheet ][ 'lDEF' ][ $field ][ 'vDEF' ] ) )
		{
			$this->_getFlexformValueDiePrompt( $method, $line, $sheet, $field );
		}

		return $arrFlexform[ 'data' ][ $sheet ][ 'lDEF' ][ $field ][ 'vDEF' ];
	}

	/**
	 * _getFlexformValueDiePrompt():
	 *
	 * @param string method
	 * @param string line
	 * @param string label of the sheet
	 * @param string label of the field
	 * @return mixed value from flexform
	 * @access protected
	 * @version 0.5.2
	 * @since 0.5.2
	 */
	private function _getFlexformValueDiePrompt( $method, $line, $sheet, $field )
	{
		$prompt = ''
						. '<h1>'
						. 'ERROR'
						. '</h1>'
						. 'Current flexform doesn\'t contain a sheet ' . $sheet . ' with field ' . $field . '<br />'
						. 'Maybe the flexform wasn\'t saved once. Please save it and try again.<br />'
						. '<br />'
						. 'Error occurs @ ' . $method . ' (#' . $line . ')'
		;
		die( $prompt );
	}

	/**
	 * Initializes this object
	 *
	 * @return void
	 * @codeCoverageIgnore
	 * @throws InvalidSlotException
	 * @throws InvalidSlotReturnException
	 */
	protected function _initObjectContent()
	{
		// @extensionScannerIgnoreLine Seems to be a false positive: getContentObject() is still correct in 9.0
		$this->contentObject = $this->configurationManager->getContentObject();
		//var_dump(__METHOD__, __LINE__, $this->contentObject->data);
	}

	/**
	 * @param AreaRepository $areaRepository
	 * @return void
	 */
	public function injectAreaRepository( AreaRepository $areaRepository )
	{
		$this->areaRepository = $areaRepository;
	}

	/**
	 * @param CategoryRepository $categoryRepository
	 * @return void
	 */
	public function injectCategoryRepository( CategoryRepository $categoryRepository )
	{
		$this->categoryRepository = $categoryRepository;
	}

	/**
	 * @param CellRepository $cellRepository
	 * @return void
	 */
	public function injectCellRepository( CellRepository $cellRepository )
	{
		$this->cellRepository = $cellRepository;
	}

	/**
	 * @param ContentRepository $contentRepository
	 * @return void
	 */
	public function injectContentRepository( ContentRepository $contentRepository )
	{
		$this->contentRepository = $contentRepository;
	}

	/**
	 * @param CssRepository $cssRepository
	 * @return void
	 */
	public function injectCssRepository( CssRepository $cssRepository )
	{
		$this->cssRepository = $cssRepository;
	}

	/**
	 * @param GroupRepository $groupRepository
	 * @return void
	 */
	public function injectGroupRepository( GroupRepository $groupRepository )
	{
		$this->groupRepository = $groupRepository;
	}

	/**
	 * @param PageRepository $pageRepository
	 * @return void
	 */
	public function injectPageRepository( PageRepository $pageRepository )
	{
		$this->pageRepository = $pageRepository;
	}

	/**
	 * @param ProductRepository $productRepository
	 * @return void
	 */
	public function injectProductRepository( ProductRepository $productRepository )
	{
		$this->productRepository = $productRepository;
	}

	/**
	 * @param TicketRepository $ticketRepository
	 * @return void
	 */
	public function injectTicketRepository( TicketRepository $ticketRepository )
	{
		$this->ticketRepository = $ticketRepository;
	}

}
