<?php

namespace Netzmacher\Tile\Controller;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * AreaController: backend module
 * 
 * @version 0.0.6
 * @since 0.0.1
 */
class AreaController extends AbstractController
{

	/**
	 * _addCells
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $newArea
	 * @return void
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	private function _addCells( \Netzmacher\Tile\Domain\Model\Area $newArea )
	{
		for( $y = 1; $y <= $newArea->getLeny(); $y++ )
		{
			for( $x = 1; $x <= $newArea->getLenx(); $x++ )
			{
				$newCell = new \Netzmacher\Tile\Domain\Model\Cell();
				$cellTitle = $newArea->getTitle() . ' x' . $x . 'y' . $y;
				$newCell->setTitle( $cellTitle );
				$newCell->setX( $x );
				$newCell->setY( $y );
				$newArea->addTxTileCell( $newCell );
			}
		}
		$newArea->addTxTileCell( $newCell );
		return $newArea;
	}

	/**
	 * _getCells
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _getCells( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		static $rows = null;

		if( is_array( $rows ) )
		{
			//$this->addFlashMessage( __METHOD__ . ' (#' . __LINE__ . ')', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
			return $rows;
		}

		//$this->addFlashMessage( __METHOD__ . ' (#' . __LINE__ . ')', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		$rows = $this->cellRepository->getCellsByArea( $area );
		return $rows;
	}

	/**
	 * _sumGroup
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @param string $relation
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _sumGroup( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$groups = [];
		$sum = 0;
		$rows = $this->_getCells( $area );
		foreach( $rows as $row )
		{
			if( !$row[ 'tx_tile_group' ] )
			{
				continue;
			}
			$group = $row[ 'tx_tile_group' ];
			if( in_array( $row[ 'tx_tile_group' ], $groups ) )
			{
				continue;
			}
			$groups[] = $group;
			$sum++;
		}
		return $sum;
	}

	/**
	 * _sumGroupProduct
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @param string $relation
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3857
	 */
	private function _sumGroupProduct( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$products = [];
		$sum = 0;
		$rows = $this->_getCells( $area );
		foreach( $rows as $row )
		{
			if( !$row[ 'tx_tile_product' ] )
			{
				continue;
			}
			$product = $row[ 'tx_tile_product' ];
			if( in_array( $row[ 'tx_tile_product' ], $products ) )
			{
				continue;
			}
			$products[] = $product;
			$sum++;
		}
		return $sum;
	}

	/**
	 * _sumRelation
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @param string $relation
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _sumRelation( \Netzmacher\Tile\Domain\Model\Area $area, $relation )
	{
		$sum = 0;
		$rows = $this->_getCells( $area );
		foreach( $rows as $row )
		{
			if( $row[ $relation ] )
			{
				$sum++;
			}
		}
		return $sum;
	}

	/**
	 * _sumTile
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return int
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _sumTile( \Netzmacher\Tile\Domain\Model\Area $area ): int
	{
		return ( int ) count( $this->_getCells( $area ) );
	}

	/**
	 * action create
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $newArea
	 * @return void
	 * @version 0.0.2
	 * @since 0.0.1
	 */
	public function createAction( \Netzmacher\Tile\Domain\Model\Area $newArea )
	{
		$newArea = $this->_addCells( $newArea );
		$this->areaRepository->add( $newArea );
//		$this->addFlashMessage( 'The object was created. Please be aware that this action is publicly accessible unless you implement an access check. See https://docs.typo3.org/typo3cms/extensions/extension_builder/User/Index.html', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		$this->redirect( 'list', 'Module' );
	}

	/**
	 * action createconfirm
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $newArea
	 * @return void
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	public function createconfirmAction( \Netzmacher\Tile\Domain\Model\Area $newArea )
	{
		$this->view->assign( 'newArea', $newArea );
	}

	/**
	 * action delete
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.1
	 * @internal #t3857
	 */
	public function deleteAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$groups = [];
		$products = [];

		$cells = $this->cellRepository->findBytx_tile_area( $area->getUid() );
		foreach( $cells as $cell )
		{
			$groups[] = $cell->getTxTileGroup();
			$products[] = $cell->getTxTileProduct();
			$area->removeTxTileCell( $cell );
			$this->cellRepository->remove( $cell );
		}
		$this->areaRepository->remove( $area );
		$this->groupRepository->removeGroups( $groups );
		$this->productRepository->removeProducts( $products );
		$this->redirect( 'list', 'Module' );
	}

	/**
	 * action deleteconfirm
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	public function deleteconfirmAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$this->view->assign( 'area', $area );
	}

	/**
	 * action edit
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @ignorevalidation $area
	 * @return void
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	public function editAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$this->view->assign( 'area', $area );
	}

	/**
	 * action new
	 * 
	 * @return void
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	public function newAction()
	{
		
	}

	/**
	 * action show
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	public function showAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$this->view->assign( 'sumCategory', $this->_sumRelation( $area, 'tx_tile_category' ) );
		$this->view->assign( 'sumGroup', $this->_sumGroup( $area ) );
		$this->view->assign( 'sumGroupProduct', $this->_sumGroupProduct( $area ) );
		$this->view->assign( 'sumTicket', $this->_sumRelation( $area, 'tx_tile_ticket' ) );
		$this->view->assign( 'sumTile', $this->_sumTile( $area ) );
		$this->view->assign( 'area', $area );
	}

	/**
	 * action update
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.1
	 * @since 0.0.1
	 */
	public function updateAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$this->addFlashMessage( 'The object was updated. Please be aware that this action is publicly accessible unless you implement an access check. See https://docs.typo3.org/typo3cms/extensions/extension_builder/User/Index.html', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		$this->areaRepository->update( $area );
		$this->redirect( 'list', 'Module' );
	}

}
