<?php

namespace Netzmacher\Tile\Controller;

use Netzmacher\Tile\Utility\Module\AreaModuleUtility;
use Netzmacher\Tile\Utility\Module\CellModuleUtility;
use Netzmacher\Tile\Utility\SessionUtility;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * BookingController: frontend plugin for booking tiles
 * 
 * @version 0.0.7
 * @since 0.0.6
 */
class BookingController extends AbstractFrontendController
{
//		/**
//	 * Content object of the Caddy
//	 * 
//	 * @var \Netzmacher\Tile\Domain\Model\Caddy
//	 */
//	private $_cObjCaddy = null;

	/**
	 * _cellIntoTheCaddy
	 * 
	 * @param int $caddyUid
	 * @return \Netzmacher\Tile\Domain\Model\Content $cObjCaddy
	 * @version 0.0.8
	 * @since 0.0.8
	 * @internal #t3869
	 */
	private function _getContentCaddy( $caddyUid )
	{
		static $cObjCaddy = [];

		if( !empty( $cObjCaddy[ $caddyUid ] ) )
		{
			return $cObjCaddy[ $caddyUid ];
		}

		$cObjCaddy[ $caddyUid ] = $this->contentRepository->findByUid( $caddyUid );
		return $cObjCaddy[ $caddyUid ];
	}

	/**
	 * _cellIntoTheCaddy
	 * 
	 * @return integer pid
	 * @version 0.9.2
	 * @since 0.0.8
	 * @internal #t3869
	 */
	private function _getCaddyPid()
	{
		$caddyUid = $this->_getFlexformCaddy();
		$content = $this->_getContentCaddy( $caddyUid );
		if( empty( $content ) )
		{
			$this->addFlashMessage( 'Die Anwendung ist mit keinem gültigen Warenkorb verbunden.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
			$this->addFlashMessage( 'Prüfe bitte den Warenkorb des Plugins.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
			return false;
		}
		return $content->getPid();
	}

	/**
	 * _cellIntoTheCaddy
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return boolean
	 * @version 0.0.8
	 * @since 0.0.8
	 * @internal #t3869
	 */
	private function _cellIntoTheCaddy( \Netzmacher\Tile\Domain\Model\Cell $cell ): bool
	{
		$caddyUid = $this->_getFlexformCaddy();
		if( empty( $caddyUid ) )
		{
			$this->addFlashMessage( 'Die Anwendung ist mit keinem Warenkorb verbunden.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
			$this->addFlashMessage( 'Aktualisiere bitte das Plugin.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
			return false;
		}

		$caddyPid = $this->_getCaddyPid( $caddyUid );
		$productUid = $cell->getTxTileProduct()->getUid();

		$conf = array(
				'parameter' => $caddyPid
				, 'additionalParams' => '&tx_browser_pi1[itemUid]=' . $productUid
				, 'useCashHash' => true
				, 'returnLast' => 'url'
				, 'forceAbsoluteUrl' => false
		);
		$uri = $this->contentObject->typoLink( 'Warenkorb', $conf );
		$this->redirectToUri( $uri );
		$link = 'Warenkorb';

		$this->addFlashMessage( 'Ticket wurde in den ' . $link . ' gelegt. #' . $caddyPid, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK );
		return true;
	}

	/**
	 * _cellIsSelled
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return boolean
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _cellIsSelled( \Netzmacher\Tile\Domain\Model\Cell $cell ): bool
	{
		$product = $cell->getTxTileProduct();
		if( $product->getStockquantity() > 0 )
		{
			return false;
		}
		$this->addFlashMessage( 'Ticket ist bereits verkauft.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
		return true;
	}

	/**
	 * _cellIsWoTicket
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return boolean
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _cellIsWoTicket( \Netzmacher\Tile\Domain\Model\Cell $cell ): bool
	{
		if( $cell->getTxTileProduct() > 0 )
		{
			return false;
		}
		$this->addFlashMessage( 'Objekt kann nicht gebucht werden: kein Ticket.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		return true;
	}

	/**
	 * _getAreaCells
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return object cells
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	private function _getAreaCells( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$cells = $this->cellRepository->findBytx_tile_area( $area->getUid() );
		return $cells;
	}

	/**
	 * _getFlexformArea
	 * 
	 * @return object area
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _getFlexformArea()
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$sheet = 'area';
		$field = 'settings.flexform.pi1.area.uid';
		$areaUid = $this->_getFlexformValue( __METHOD__, __LINE__, $xmlFlexform, $sheet, $field );
		if( !empty( $areaUid ) )
		{
			$area = $this->areaRepository->findByUid( $areaUid );
			return $area;
		}
		$this->addFlashMessage( 'Die Anwendung ist mit keinem gültigen Plan verbunden.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
		$this->addFlashMessage( 'Prüfe bitte das Plugin.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
//		$prompt = ''
//						. '<h1>'
//						. 'ERROR: site plan is missing'
//						. '</h1>'
//						. 'Current flexform isn\'t linked with an area (site plan).<br />'
//						. 'Please take care of a proper configuration.<br />'
//						. '<br />'
//						. 'Error occurs @ ' . __METHOD__ . ' (#' . __LINE__ . ')'
//		;
//		die( $prompt );
	}

	/**
	 * _getFlexformCaddy
	 * 
	 * @return integer
	 * @version 0.0.8
	 * @since 0.0.8
	 */
	private function _getFlexformCaddy()
	{
		static $caddyUid = null;

		if( !empty( $caddyUid ) )
		{
			return $caddyUid;
		}

		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$sheet = 'caddy';
		$field = 'settings.flexform.pi1.caddy.uid';
		$caddyUid = $this->_getFlexformValue( __METHOD__, __LINE__, $xmlFlexform, $sheet, $field );
		if( !empty( $caddyUid ) )
		{
			return $caddyUid;
		}
		$this->addFlashMessage( 'Die Anwendung ist mit keinem Warenkorb verbunden.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
		$this->addFlashMessage( 'Prüfe bitte das Plugin.', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
//		$prompt = ''
//						. '<h1>'
//						. 'ERROR: caddy is missing'
//						. '</h1>'
//						. 'Current flexform isn\'t linked with a Caddy.<br />'
//						. 'Please take care of a proper configuration.<br />'
//						. '<br />'
//						. 'Error occurs @ ' . __METHOD__ . ' (#' . __LINE__ . ')'
//		;
//		die( $prompt );
	}

	/**
	 * _getFlexformCellsize
	 * 
	 * @param int $default Default size
	 * @return integer
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _getFlexformCellsize( $default = 15 )
	{
		$xmlFlexform = $this->contentObject->data[ 'pi_flexform' ];
		$sheet = 'area';
		$field = 'settings.flexform.pi1.area.cellsize';
		$cellsize = $this->_getFlexformValue( __METHOD__, __LINE__, $xmlFlexform, $sheet, $field );
		if( !empty( $cellsize ) )
		{
			return $cellsize;
		}
		return $default;
	}

	/**
	 * caddy
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.8
	 * @since 0.0.7
	 * @internal #t3869
	 */
	public function caddyAction( \Netzmacher\Tile\Domain\Model\Cell $cell, \Netzmacher\Tile\Domain\Model\Area $area )
	{
		switch( true )
		{
			case($this->_cellIsWoTicket( $cell )):
				break;
			case($this->_cellIsSelled( $cell )):
				break;
			case($this->_cellIntoTheCaddy( $cell )):
				break;
			default:
				$this->addFlashMessage( 'Undefinierter Fehler: Es wurde keine Bedingung erfüllt @ ' . __METHOD__ . ' (#' . __LINE__ . ')', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
				break;
		}

		$arguments = [
				'area' => $area
		];
		$this->redirect( 'list', NULL, NULL, $arguments );
	}

	/**
	 * initializeAction: is executed before any Action in the Controller.
	 * 
	 * @return void
	 * @version 0.0.7
	 * @since 0.0.6
	 */
	public function initializeAction()
	{
		$this->_initObjects();
		$this->_oSessionUtility->setCData( $this->contentObject->data );
		$this->_oSessionUtility->setSession();
	}

	/**
	 * listAction
	 * 
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	public function listAction()
	{
		$this->view->assignMultiple(
						array(
								'caddyPid' => $this->_getCaddyPid()
								, 'cData' => $this->contentObject->data
								, 'formCellWiBgimageChecked' => $this->_oSessionUtility->getSessionElement( 'formCellWiBgimage', true )
								, 'formCellWiCategoriesChecked' => $this->_oSessionUtility->getSessionElement( 'formCellWiCategories', true )
								, 'formCellWiGroupsChecked' => $this->_oSessionUtility->getSessionElement( 'formCellWiGroups', true )
								, 'formCellWiTicketsChecked' => $this->_oSessionUtility->getSessionElement( 'formCellWiTickets', true )
								, 'formCellWiUndefinedCellsChecked' => $this->_oSessionUtility->getSessionElement( 'formCellWiUndefinedCells', true )
								, 'size' => $this->_getFlexformCellsize()
						)
		);

		$area = $this->_getFlexformArea();

		if( empty( $area ) )
		{
			return;
		}
		$this->view->assignMultiple(
						array(
								'area' => $area
								, 'cells' => $this->_getAreaCells( $area )
								, 'grid-template-columns' => AreaModuleUtility::getAutoElementsForGridCSS( $area )
						)
		);
	}

}
