<?php

namespace Netzmacher\Tile\Controller;

use Netzmacher\Tile\Utility\Module\AreaModuleUtility;
use Netzmacher\Tile\Utility\Module\CellModuleUtility;
use Netzmacher\Tile\Utility\Module\SessionModuleUtility;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * CategoryController: backend module
 * 
 * @version 0.0.6
 * @since 0.0.1
 */
class CategoryController extends AbstractController
{

	/**
	 * _getAreaCells
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	private function _getAreaCells( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$cells = $this->cellRepository->findBytx_tile_area( $area->getUid() );
		return $cells;
	}

	/**
	 * _getCategories()
	 * 
	 * @return \Netzmacher\Tile\Domain\Model\Category 
	 * @version 0.0.3
	 * @since 0.0.3
	 * @internal #t3729
	 */
	private function _getCategories()
	{
		$returnArray = [];
		$defaultOrderings = [
				'title' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING,
		];
		$this->categoryRepository->setDefaultOrderings( $defaultOrderings );
		$categories = $this->categoryRepository->findAll();

		foreach( $categories as $category )
		{
			$returnArray[ $category->getUid() ] = $category->getTitle();
		}

		return $returnArray;
	}

	/**
	 * _getCategoryTitle()
	 * 
	 * @param integer	$uid
	 * 
	 * @return \Netzmacher\Tile\Domain\Model\Category 
	 * @version 0.0.5
	 * @since 0.0.5
	 * @internal #t3749
	 */
	private function _getCategoryTitle( $uid )
	{
		$category = $this->categoryRepository->findByUid( $uid );
		if( !is_object( $category ) )
		{
			return;
		}

		return $category->getTitle();
	}

	/**
	 * _setCategoryForCellRange()
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * 
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.5
	 * @internal #t3749
	 */
	private function _setCategoryForCellRange( \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		if( !$this->_setCategoryForCellRangeRequirements() )
		{
			return;
		}

		$formerCellUid = SessionModuleUtility::getValue( 'formerCellUid' );

		// return, there is no former cell
		if( empty( $formerCellUid ) )
		{
//			$prompt = 'No former cell @ ' . __METHOD__ . ' (#' . __LINE__ . ')';
//			$this->addFlashMessage( $prompt, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
			SessionModuleUtility::setValue( 'formerCellUid', $cell->getUid() );
			return;
		}

		$formerCell = $this->cellRepository->findByUid( $formerCellUid );
		SessionModuleUtility::removeValue( 'formerCellUid' );

		$xyRange = CellModuleUtility::getXsYsOrdered( $formerCell, $cell );
		$cellUids = $this->cellRepository->getCellsByXyRange( $cell, $xyRange );
		foreach( $cellUids as $cellUid )
		{
			$cell = $this->cellRepository->findByUid( $cellUid );
			$cell->setTxTileCategory( SessionModuleUtility::getFormCellCategory() );
			$this->cellRepository->update( $cell );
		}

		return;

//		$prompt = 'x from ' . $xyRange[ 'minX' ] . ' to ' . $xyRange[ 'maxX' ] . ', y from ' . $xyRange[ 'minY' ] . ' to ' . $xyRange[ 'maxY' ];
//		$this->addFlashMessage( $prompt, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
//		$prompt = 'former cell: ' . $formerCell->getUid() . ', x: ' . $formerCell->getX() . ', y: ' . $formerCell->getY() . ' @ ' . __METHOD__ . ' (#' . __LINE__ . ')';
//		$this->addFlashMessage( $prompt, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
//		$prompt = 'cell: ' . $cell->getUid() . ', x: ' . $cell->getX() . ', y: ' . $cell->getY() . ' @ ' . __METHOD__ . ' (#' . __LINE__ . ')';
//		$this->addFlashMessage( $prompt, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
	}

	/**
	 * _setCategoryForCellRangeRequirements()
	 * 
	 * @return boolean
	 * @version 0.0.6
	 * @since 0.0.5
	 * @internal #t3749
	 */
	private function _setCategoryForCellRangeRequirements()
	{
		$mode = SessionModuleUtility::getFormCellMode();
		switch( $mode )
		{
			case('cell'):
				return false;
			case('group'):
				return true;
			default:
				$prompt = 'Warning @ ' . __METHOD__ . ' (#' . __LINE__ . ')! Undefined value in switch: "' . $mode . '"';
				$this->addFlashMessage( $prompt, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
				return false;
		}
	}

	/**
	 * action enableAction
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.5
	 * @since 0.0.5
	 * @internal #i0001
	 */
	public function enableAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$categoryTitle = $this->_getCategoryTitle( SessionModuleUtility::getFormCellCategory() );
		if( empty( $categoryTitle ) )
		{
			$categoryTitle = 'Zurücksetzen';
		}
		$formCellCategoryOptions = [ 0 => 'Zurücksetzen' ] + ( array ) $this->_getCategories();
		$formCellCategorySelected = SessionModuleUtility::getFormCellCategory();
		$formCellModeSelected = SessionModuleUtility::getFormCellMode();
		if( empty( $formCellModeSelected ) )
		{
			$formCellModeSelected = 'cell';
		}
		$formCellModeOptions = [
				'cell' => 'Kachel',
				'group' => 'Gruppe'
		];
		$currentModeLabel = $formCellModeOptions[ SessionModuleUtility::getFormCellMode() ];
		if( empty( $currentModeLabel ) )
		{
			$currentModeLabel = 'Kachel';
		}

		$formCellWiBgimageChecked = SessionModuleUtility::getFormCellWiBgimage( true );
		$formCellWiCategoriesChecked = SessionModuleUtility::getFormCellWiCategories( true );
		$formCellWiGroupsChecked = SessionModuleUtility::getFormCellWiGroups( true );
		$formCellWiTicketsChecked = SessionModuleUtility::getFormCellWiTickets( true );
		$formCellWiUndefinedCellsChecked = SessionModuleUtility::getFormCellWiUndefinedCells( true );

		$this->view->assign( 'area', $area );
		$this->view->assign( 'categoryTitle', $categoryTitle );
		$this->view->assign( 'cells', $this->_getAreaCells( $area ) );
		$this->view->assign( 'currentModeLabel', $currentModeLabel );
		$this->view->assign( 'formCellCategoryOptions', $formCellCategoryOptions );
		$this->view->assign( 'formCellCategorySelected', $formCellCategorySelected );
		$this->view->assign( 'formCellModeOptions', $formCellModeOptions );
		$this->view->assign( 'formCellModeSelected', $formCellModeSelected );
		$this->view->assign( 'formCellWiBgimageChecked', $formCellWiBgimageChecked );
		$this->view->assign( 'formCellWiCategoriesChecked', $formCellWiCategoriesChecked );
		$this->view->assign( 'formCellWiGroupsChecked', $formCellWiGroupsChecked );
		$this->view->assign( 'formCellWiTicketsChecked', $formCellWiTicketsChecked );
		$this->view->assign( 'formCellWiUndefinedCellsChecked', $formCellWiUndefinedCellsChecked );
		$this->view->assign( 'grid-template-columns', AreaModuleUtility::getAutoElementsForGridCSS( $area ) );
		$this->view->assign( 'size', '15' );
	}

	/**
	 * initializeAction: is executed before any Action in the Controller.
	 * 
	 * @return void
	 * @version 0.0.3
	 * @since 0.0.3
	 * @internal #t3728
	 */
	public function initializeAction()
	{
		SessionModuleUtility::setSession();
	}
	
	/**
	 * action updatecell
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.1
	 */
	public function updatecellAction( \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		//$this->addFlashMessage( 'The object was updated. Please be aware that this action is publicly accessible unless you implement an access check. See https://docs.typo3.org/typo3cms/extensions/extension_builder/User/Index.html', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		$this->_setCategoryForCellRange( $cell );
		$cell->setTxTileCategory( SessionModuleUtility::getFormCellCategory() );
		$this->cellRepository->update( $cell );
		$area = $this->request->getArgument( 'area' );
		$arguments = [
				'area' => $area
		];
		$this->redirect( 'enable', NULL, NULL, $arguments );
	}
}
