<?php

namespace Netzmacher\Tile\Controller;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * ModuleController: backend module
 * 
 * @version 0.0.6
 * @since 0.0.1
 */
class ModuleController extends AbstractController
{

	/**
	 * _checkCss
	 * 
	 * @return boolean
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #3824
	 */
	private function _check()
	{
		switch( true )
		{
			case(!$this->_checkCss() ):
				return false;
			case(!$this->_checkCategory() ):
				return false;
			case(!$this->_checkTicket() ):
				return false;
			default:
				return true;
		}
	}

	/**
	 * _checkCategory
	 * 
	 * @return boolean
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #3824
	 */
	private function _checkCategory()
	{
		$category = $this->categoryRepository->findAll();
		if( !empty( $category ) )
		{
			return true;
		}

		$header = 'Objekt-Datensatz fehlt';
		$prompt = 'Um einen Plan zu erstellen, benötigst Du mindestens ein Objekt. '
						. 'Füge dieses bitte mit dem Modul "Web > Liste" hinzu.';
		$this->addFlashMessage( $prompt, $header, \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		return false;
	}

	/**
	 * _checkCss
	 * 
	 * @return boolean
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #3824
	 */
	private function _checkCss()
	{
		$css = $this->cssRepository->findAll();
		if( !empty( $css ) )
		{
			return true;
		}

		$header = 'CSS-Datensatz fehlt';
		$prompt = 'Um einen Plan zu erstellen, benötigst Du mindestens einen CSS-Datensatz. '
						. 'Füge diesen bitte mit dem Modul "Web > Liste" hinzu.';
		$this->addFlashMessage( $prompt, $header, \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		return false;
	}

	/**
	 * _checkTicket
	 * 
	 * @return boolean
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #3824
	 */
	private function _checkTicket()
	{
		$ticket = $this->ticketRepository->findAll();
		if( !empty( $ticket ) )
		{
			return true;
		}

		$header = 'Ticket-Datensatz fehlt';
		$prompt = 'Um einen Plan zu erstellen, benötigst Du mindestens ein Ticket. '
						. 'Füge diesen bitte mit dem Modul "Web > Liste" hinzu.';
		$this->addFlashMessage( $prompt, $header, \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		return false;
	}

	/**
	 * _gridTemplateColumns
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.2
	 * @since 0.0.2
	 */
	private function _gridTemplateColumns( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$arrAuto = null;
		for( $x = 1; $x <= $area->getLenx(); $x++ )
		{
			$arrAuto[] = 'auto';
		}
		$strAuto = implode( ' ', ( array ) $arrAuto );
		return $strAuto;
	}

	/**
	 * action copy
	 * 
	 * @return void
	 * @version 0.0.2
	 * @since 0.0.2
	 */
	public function copyAction()
	{
		$this->addFlashMessage( 'Die Aktion [Kopieren] ist nicht implementiert. ', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
		$this->redirect( 'list' );
	}

	/**
	 * action list
	 * 
	 * @return void
	 * @version 0.0.2
	 * @since 0.0.1
	 */
	public function listAction()
	{
		$this->_check();

		$defaultOrderings = [
				'title' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING,
		];
		$this->areaRepository->setDefaultOrderings( $defaultOrderings );
		$areas = $this->areaRepository->findAll();
		$this->view->assign( 'areas', $areas );
	}

}
