<?php

namespace Netzmacher\Tile\Controller;

use Netzmacher\Tile\Utility\Module\SessionModuleUtility;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * ShopController: backend module
 * 
 * @version 0.9.4
 * @since 0.0.1
 * @internal #t3823
 */
class ShopController extends AbstractController
{

	/**
	 * _check
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return bool
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _check( \Netzmacher\Tile\Domain\Model\Area $area ): bool
	{
		switch( true )
		{
			case(!$this->_checkGroup( $area ) ):
				return false;
			default:
				return true;
		}
	}

	/**
	 * _checkGroup
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return bool
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _checkGroup( \Netzmacher\Tile\Domain\Model\Area $area ): bool
	{
		$cells = $this->cellRepository->findBytx_tile_area( $area->getUid() );
		foreach( $cells as $cell )
		{
			if( $cell->getTxTileGroup() )
			{
				return true;
			}
		}

		$header = 'Ticket-Datensatz fehlt';
		$prompt = 'Um mit einem Plan live zu gehen, muss mindesten eine Kachel mit einem Ticket verbunden sein. '
						. 'Bearbeite den ausgewählten Plan bitte entsprechend.';
		$this->addFlashMessage( $prompt, $header, \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
		return false;
	}

	/**
	 * _checkPid
	 * 
	 * @param int $pid
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3843
	 */
	private function _checkPid( $pid )
	{
		$checkPid = true;

		switch( true )
		{
			case($pid < 1):
				$checkPid = false;
				break;
			case(!$this->pageRepository->findByUid( $pid )):
				$checkPid = false;
				break;
		}

		if( $checkPid )
		{
			return true;
		}

		$header = '';
		$prompt = 'Die Page ID "' . $pid . '" für den Ziel-Ordner ist nicht in Ordnung.';
		$this->addFlashMessage( $prompt, $header, \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );

		return false;
	}

	/**
	 * _checkTitle
	 * 
	 * @param string $title
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3863
	 */
	private function _checkTitle( $title )
	{
		if( !empty( $title ) )
		{
			return true;
		}

		$header = '';
		$prompt = 'Datum der Veranstaltung ist nicht angegeben.';
		$this->addFlashMessage( $prompt, $header, \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );

		return false;
	}

	/**
	 * _cloneArea
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @param int $pid
	 * @return int
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3843
	 */
	private function _cloneArea( \Netzmacher\Tile\Domain\Model\Area $area, $pid ): int
	{
		$clone = new \Netzmacher\Tile\Domain\Model\Area;

		$properties = $area->_getProperties();

		// uid, pid
		unset( $properties[ 'uid' ] );

		// type
		$properties[ 'displaycell' ] = 0; // clone
		$properties[ 'pid' ] = $pid;
		$properties[ 'type' ] = 1; // clone
		// image
		// #t3849, 190702, dwildt: next line doesn't work
		//$clone->setImage( $area->getImage() );
		unset( $properties[ 'image' ] );

		foreach( $properties as $key => $value )
		{
			$clone->_setProperty( $key, $value );
		}

		$clone->setTitle( $clone->getTitle() . ' – cloned at ' . strftime( '%c', time() ) );
		$this->areaRepository->add( $clone );
		$this->persistenceManager->persistAll();
		$uid = $clone->getUid();
		$page = $this->pageRepository->findByUid( $pid );
		$prompt = $clone->getTitle() . ' (#' . $uid . ') was cloned at page ' . $page->getTitle() . ' (#' . $clone->getPid() . ')';
		$this->addFlashMessage( $prompt, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK );

		return $uid;
	}

	/**
	 * _cloneCell
	 * 
	 * @param int $pid
	 * @param int $cloneAreaUid
	 * @param array $properties
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3845
	 */
	private function _cloneCell( $pid, $cloneAreaUid, $properties )
	{
		$clone = new \Netzmacher\Tile\Domain\Model\Cell;

		// uid, pid
		unset( $properties[ 'uid' ] );
		$properties[ 'pid' ] = $pid;

		// Area
		$clone->setTxTileArea( $this->areaRepository->findByUid( $cloneAreaUid ) );
		unset( $properties[ 'tx_tile_area' ] );

		// Category
		$clone->setTxTileCategory( $properties[ 'tx_tile_category' ] );
		unset( $properties[ 'tx_tile_category' ] );

		// Ticket (must be before product!
		$clone->setTxTileTicket( $properties[ 'tx_tile_ticket' ] );
		unset( $properties[ 'tx_tile_ticket' ] );

		// Group and Product
		$groupClone = $this->_cloneCellGroup( $properties[ 'tx_tile_group' ], $pid );
		if( $groupClone )
		{
			$clone = $this->_createProductByGroup( $clone, $groupClone, $pid );
			$clone->setTxTileGroup( $groupClone );
		}
		unset( $properties[ 'tx_tile_group' ] );
		unset( $properties[ 'tx_tile_product' ] );

		// all other properties
		foreach( $properties as $key => $value )
		{
			$clone->_setProperty( $key, $value );
		}

		$this->cellRepository->add( $clone );
	}

	/**
	 * _cloneCellGroup
	 * 
	 * @param int $groupUid
	 * @param int $pid
	 * @return mixed
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3848
	 */
	private function _cloneCellGroup( $groupOrigUid, $pid )
	{
		static $groups = []; // originalUid => cloneUid

		if( empty( $groupOrigUid ) )
		{
			return null;
		}

		if( in_array( $groupOrigUid, array_keys( $groups ) ) )
		{
			$groupCloneUid = $groups[ $groupOrigUid ];
			return $this->groupRepository->findByUid( $groupCloneUid );
		}

		$groupOrig = $this->groupRepository->findByUid( $groupOrigUid );

		$groupClone = new \Netzmacher\Tile\Domain\Model\Group;
		$groupClone->_setProperty( 'pid', $pid );
		$groupClone->setTitle( $groupOrig->getTitle() );
		$groupClone->setTxTileCell( $groupOrig->getTxTileCell() );

		$this->groupRepository->add( $groupClone );
		$this->persistenceManager->persistAll();

		$groupCloneUid = $groupClone->getUid();
		$groups[ $groupOrigUid ] = $groupCloneUid;

		//$this->addFlashMessage( __METHOD__ . ' (#' . __LINE__ . '): Group added with uid (#' . $groupCloneUid . ')', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );

		return $this->groupRepository->findByUid( $groupCloneUid );
	}

	/**
	 * _createProductByGroup
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @param \Netzmacher\Tile\Domain\Model\Group $group
	 * @param int $pid
	 * @return \Netzmacher\Tile\Domain\Model\Cell
	 * @version 0.9.4
	 * @since 0.0.6
	 * @internal #t3857
	 */
	private function _createProductByGroup( \Netzmacher\Tile\Domain\Model\Cell $cell, \Netzmacher\Tile\Domain\Model\Group $group, $pid ): \Netzmacher\Tile\Domain\Model\Cell
	{
		static $products = []; // groupUid => cloneUid

		if( empty( $group ) )
		{
			return $cell;
		}

		$groupUid = $group->getUid();
		if( in_array( $groupUid, array_keys( $products ) ) )
		{
			$productUid = $products[ $groupUid ];
			$product = $this->productRepository->findByUid( $productUid );
			$cell->setTxTileProduct( $product );
			return $cell;
		}

		$product = new \Netzmacher\Tile\Domain\Model\Product;
		$product->_setProperty( 'pid', $pid );
		$ticketUid = $cell->getTxTileTicket();
		$ticket = $this->ticketRepository->findByUid( $ticketUid );
		$product->setPrice( $ticket->getPrice() );
		$product->setStockmanagement( true );
		$product->setStockquantity( 1 );
		$product->setTax( $ticket->getTax() );
		$title = SessionModuleUtility::getFormShopTitle() . ': ' . $group->getTitle();
		$product->setTitle( $title );

		$this->productRepository->add( $product );
		$this->persistenceManager->persistAll();

		$productUid = $product->getUid();
		$products[ $groupUid ] = $productUid;
		$cell->setTxTileProduct( $product );
		return $cell;
	}

	/**
	 * _cloneCells
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @param int $pid
	 * @param int $cloneAreaUid
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3845
	 */
	private function _cloneCells( $area, $pid, $cloneAreaUid )
	{
		$aCells = $this->cellRepository->getCellsByArea( $area );
		foreach( $aCells as $aCell )
		{
			$this->_cloneCell( $pid, $cloneAreaUid, $aCell );
		}
	}

	/**
	 * _getCells
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _getCells( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		static $rows = null;

		if( is_array( $rows ) )
		{
			//$this->addFlashMessage( __METHOD__ . ' (#' . __LINE__ . ')', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
			return $rows;
		}

		//$this->addFlashMessage( __METHOD__ . ' (#' . __LINE__ . ')', '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
		$rows = $this->cellRepository->getCellsByArea( $area );
		return $rows;
	}

	/**
	 * _sumGroup
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @param string $relation
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _sumGroup( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$groups = [];
		$sum = 0;
		$rows = $this->_getCells( $area );
		foreach( $rows as $row )
		{
			if( !$row[ 'tx_tile_group' ] )
			{
				continue;
			}
			$group = $row[ 'tx_tile_group' ];
			if( in_array( $row[ 'tx_tile_group' ], $groups ) )
			{
				continue;
			}
			$groups[] = $group;
			$sum++;
		}
		return $sum;
	}

	/**
	 * _sumRelation
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @param string $relation
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _sumRelation( \Netzmacher\Tile\Domain\Model\Area $area, $relation )
	{
		$sum = 0;
		$rows = $this->_getCells( $area );
		foreach( $rows as $row )
		{
			if( $row[ $relation ] )
			{
				$sum++;
			}
		}
		return $sum;
	}

	/**
	 * action createAction
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812, #t3843
	 */
	public function createAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$pid = ( int ) SessionModuleUtility::getFormShopDestpid();
		if( !$this->_checkPid( $pid ) )
		{
			$this->redirect( 'createconfirm', 'Shop', null, array( 'area' => $area ) );
		}

		$title = SessionModuleUtility::getFormShopTitle();
		if( !$this->_checkTitle( $title ) )
		{
			$this->redirect( 'createconfirm', 'Shop', null, array( 'area' => $area ) );
		}

		$cloneAreaUid = $this->_cloneArea( $area, $pid );
		$cloneCells = $this->_cloneCells( $area, $pid, $cloneAreaUid );

		$this->redirect( 'list', 'Module' );
	}

	/**
	 * action createconfirmAction
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	public function createconfirmAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		if( !$this->_check( $area ) )
		{
			return;
		}
		$formShopDestpid = SessionModuleUtility::getFormShopDestpid();
		$formShopTitle = SessionModuleUtility::getFormShopTitle();
		$this->view->assign( 'formShopDestpid', $formShopDestpid );
		$this->view->assign( 'formShopTitle', $formShopTitle );
		$this->view->assign( 'sumCategory', $this->_sumRelation( $area, 'tx_tile_category' ) );
		$this->view->assign( 'sumGroup', $this->_sumGroup( $area ) );
		$this->view->assign( 'sumTicket', $this->_sumRelation( $area, 'tx_tile_ticket' ) );
		$this->view->assign( 'area', $area );
	}

	/**
	 * initializeAction: is executed before any Action in the Controller.
	 * 
	 * @return void
	 * @version 0.0.3
	 * @since 0.0.3
	 * @internal #t3728
	 */
	public function initializeAction()
	{
		SessionModuleUtility::setSession();
	}

}
