<?php

namespace Netzmacher\Tile\Controller;

use Netzmacher\Tile\Utility\Module\AreaModuleUtility;
use Netzmacher\Tile\Utility\Module\CellModuleUtility;
use Netzmacher\Tile\Utility\Module\SessionModuleUtility;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * TicketController: backend module
 * 
 * @version 0.0.6
 * @since 0.0.1
 */
class TicketController extends AbstractController
{

	/**
	 * _cellIsEmpty
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _cellIsEmpty( \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		switch( true )
		{
			case($cell->getTxTileGroup());
			case($cell->getTxTileTicket());
				return false;
			default:
				return true;
		}
	}

	/**
	 * _cellModeIsGroup()
	 * 
	 * @return boolean
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3816
	 */
	private function _cellModeIsGroup()
	{
		$mode = SessionModuleUtility::getFormCellMode();
		switch( $mode )
		{
			case('cell'):
				return false;
			case('group'):
				return true;
			default:
				$prompt = 'Warning @ ' . __METHOD__ . ' (#' . __LINE__ . ')! Undefined value in switch: "' . $mode . '"';
				$this->addFlashMessage( $prompt, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING );
				return false;
		}
	}

	/**
	 * _cellRangeIsEditable
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _cellRangeIsEditable( \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		switch( true )
		{
			case($this->_ticketIsNull()):
				return true;
			case($this->_cellRangeIsEmpty( $cell )):
				return true;
			default:
				return false;
		}
	}

	/**
	 * _cellRangeIsEmpty
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _cellRangeIsEmpty( \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		// Mode: single cell
		if( !$this->_cellModeIsGroup() )
		{
			if( $this->_cellIsEmpty( $cell ) )
			{
				return true;
			}
			return false;
		}

		// Mode: cell group

		$formerCellUid = SessionModuleUtility::getValue( 'formerCellUid' );
		// return, there is no former cell
		if( empty( $formerCellUid ) )
		{
			return true;
		}

		$formerCell = $this->cellRepository->findByUid( $formerCellUid );
		$xyRange = CellModuleUtility::getXsYsOrdered( $formerCell, $cell );
		$cellUids = $this->cellRepository->getCellsByXyRange( $cell, $xyRange );
		foreach( $cellUids as $cellUid )
		{
			$cell = $this->cellRepository->findByUid( $cellUid );
			if( !$this->_cellIsEmpty( $cell ) )
			{
				return false;
			}
		}

		return true;
	}

	/**
	 * _getAreaCells
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	private function _getAreaCells( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$cells = $this->cellRepository->findBytx_tile_area( $area->getUid() );
		return $cells;
	}

	/**
	 * _getTicketTitle()
	 * 
	 * @param integer	$uid
	 * 
	 * @return \Netzmacher\Tile\Domain\Model\Ticket 
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	private function _getTicketTitle( $uid )
	{
		$ticket = $this->ticketRepository->findByUid( $uid );
		if( !is_object( $ticket ) )
		{
			return;
		}

		return $ticket->getTitle();
	}

	/**
	 * _getTickets()
	 * 
	 * @return \Netzmacher\Tile\Domain\Model\Category 
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	private function _getTickets()
	{
		$returnArray = [];
		$defaultOrderings = [
				'title' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING,
		];
		$this->ticketRepository->setDefaultOrderings( $defaultOrderings );
		$tickets = $this->ticketRepository->findAll();

		foreach( $tickets as $ticket )
		{
			$returnArray[ $ticket->getUid() ] = $ticket->getTitle();
		}

		return $returnArray;
	}

	/**
	 * _promptCellRangeIsNotEmpty
	 * 
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _promptCellRangeIsNotEmpty()
	{
		$header = '';
		$prompt = 'Tickets können nicht überschrieben werden.';
		$this->addFlashMessage( $prompt, $header, \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
		$header = '';
		$prompt = 'Bitte leere den Bereich, bevor Du diesem ein neues Ticket zuweist.';
		$this->addFlashMessage( $prompt, $header, \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO );
	}

	/**
	 * _resetCellsByDeprecatedGroups()
	 * 
	 * @param array $groups
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3816
	 */
	private function _resetCellsByDeprecatedGroups( array $groups )
	{
		switch( true )
		{
			case(empty( $groups ) ):
				$prompt = 'Warning @ ' . __METHOD__ . ' (#' . __LINE__ . ')! Groups are empty.';
				$this->addFlashMessage( $prompt, 'ERROR', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
				return;
			case(!$this->_ticketIsNull() ):
				// Do nothing
				return;
			case( $this->_ticketIsNull() ):
				$groups = array_unique( $groups );
				foreach( $groups as $group )
				{
					if( empty( $group ) )
					{
						continue;
					}
					$cellUids = $this->cellRepository->getCellsByGroup( $group );
					foreach( $cellUids as $cellUid )
					{
						$cell = $this->cellRepository->findByUid( $cellUid );
						$this->_setTicketForCell( $cell );
						$this->_setGroupForCell( $cell, $group );
						$this->cellRepository->update( $cell );
					}
				}
				return;
		}
	}

	/**
	 * _removeDeprecatedGroups()
	 * 
	 * @param array $groups
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3816
	 */
	private function _removeDeprecatedGroups( array $groups )
	{
		switch( true )
		{
			case(empty( $groups ) ):
				$prompt = 'Warning @ ' . __METHOD__ . ' (#' . __LINE__ . ')! Groups are empty.';
				$this->addFlashMessage( $prompt, 'ERROR', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR );
				return;
			case(!$this->_ticketIsNull() ):
				// Do nothing
				return;
			case( $this->_ticketIsNull() ):
				$this->groupRepository->removeGroups( $groups );
				return;
		}
	}

	/**
	 * _setGroupForCell()
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @param \Netzmacher\Tile\Domain\Model\Group $group
	 * @return mixed
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3816
	 */
	private function _setGroupForCell( \Netzmacher\Tile\Domain\Model\Cell $cell, \Netzmacher\Tile\Domain\Model\Group $group = null )
	{
		switch( true )
		{
			case(!$this->_ticketIsNull() ):
				if( empty( $group ) )
				{
					// Create a new group and set it
					$group = new \Netzmacher\Tile\Domain\Model\Group();
					$title = $cell->getTitle();
					$group->setTitle( $title );
				}
				$group->setTxTileCell( ( int ) $group->getTxTileCell() + 1 );
				$cell->setTxTileGroup( $group );
				break;
			case( $this->_ticketIsNull() ):
				// Remove the group of the current cell from the current cell
				$group = $cell->getTxTileGroup();
				$cell->removeTxTileGroup( $group );
				break;
		}
		return $group;
	}

	/**
	 * _setGroupTitle()
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Group $group
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @param array $xyRange
	 * @return mixed
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3816
	 */
	private function _setGroupTitle( $cell, $xyRange, $group = null )
	{
		if( empty( $group ) )
		{
			return;
		}
		$range = 'x' . $xyRange[ 'minX' ] . 'y' . $xyRange[ 'minY' ] . '-x' . $xyRange[ 'maxX' ] . 'y' . $xyRange[ 'maxY' ];
		$title = $cell->getTxTileArea()->getTitle() . ' ' . $range;
		$group->setTitle( $title );
	}

	/**
	 * _setTicketForCell()
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * 
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3816
	 */
	private function _setTicketForCell( \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		$cell->setTxTileTicket( SessionModuleUtility::getFormCellTicket() );
		$cell->setGroupcss( SessionModuleUtility::getFormCellMode() );
		$this->cellRepository->update( $cell );
		return;
	}

	/**
	 * _setTicketForCells()
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * 
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3816
	 */
	private function _setTicketForCells( \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		$group = $this->_setGroupForCell( $cell );
		$groups[] = $group;

		if( !$this->_cellModeIsGroup() )
		{
			$this->_setTicketForCell( $cell );
			$this->_resetCellsByDeprecatedGroups( $groups );
			$this->_removeDeprecatedGroups( $groups );
			return;
		}

		$formerCellUid = SessionModuleUtility::getValue( 'formerCellUid' );

		// return: there is no former cell, any range isn't possible
		if( empty( $formerCellUid ) )
		{
			SessionModuleUtility::setValue( 'formerCellUid', $cell->getUid() );
			return;
		}

		$formerCell = $this->cellRepository->findByUid( $formerCellUid );
		SessionModuleUtility::removeValue( 'formerCellUid' );

		$xyRange = CellModuleUtility::getXsYsOrdered( $formerCell, $cell );
		$cellUids = $this->cellRepository->getCellsByXyRange( $cell, $xyRange );

		foreach( $cellUids as $cellUid )
		{
			$cell = $this->cellRepository->findByUid( $cellUid );
			$cell->setTxTileTicket( SessionModuleUtility::getFormCellTicket() );
			$cell->setGroupcss( SessionModuleUtility::getFormCellMode(), $xyRange, $cell );
			$groups[] = $this->_setGroupForCell( $cell, $group );
			$this->cellRepository->update( $cell );
		}

		$this->_setGroupTitle( $cell, $xyRange, $group );
		if( !empty( $group ) )
		{
			$group->setTxTileCell( ( int ) $group->getTxTileCell() - 1 );
		}
		$this->_resetCellsByDeprecatedGroups( $groups );
		$this->_removeDeprecatedGroups( $groups );
	}

	/**
	 * _ticketIsNull
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 */
	private function _ticketIsNull()
	{
		if( SessionModuleUtility::getFormCellTicket() == 0 )
		{
			return true;
		}
		return false;
	}

	/**
	 * action enableAction
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	public function enableAction( \Netzmacher\Tile\Domain\Model\Area $area )
	{
		$ticketTitle = $this->_getTicketTitle( SessionModuleUtility::getFormCellTicket() );
		if( empty( $ticketTitle ) )
		{
			$ticketTitle = 'Zurücksetzen';
		}
		$formCellModeSelected = SessionModuleUtility::getFormCellMode();
		if( empty( $formCellModeSelected ) )
		{
			$formCellModeSelected = 'cell';
		}
		$formCellModeOptions = [
				'cell' => 'Kachel',
				'group' => 'Gruppe'
		];
		$currentModeLabel = $formCellModeOptions[ SessionModuleUtility::getFormCellMode() ];
		if( empty( $currentModeLabel ) )
		{
			$currentModeLabel = 'Kachel';
		}

		$formCellTicketOptions = [ 0 => 'Zurücksetzen' ] + ( array ) $this->_getTickets();
		$formCellTicketSelected = SessionModuleUtility::getFormCellTicket();

		$formCellWiBgimageChecked = SessionModuleUtility::getFormCellWiBgimage( true );
		$formCellWiCategoriesChecked = SessionModuleUtility::getFormCellWiCategories( true );
		$formCellWiGroupsChecked = SessionModuleUtility::getFormCellWiGroups( true );
		$formCellWiTicketsChecked = SessionModuleUtility::getFormCellWiTickets( true );
		$formCellWiUndefinedCellsChecked = SessionModuleUtility::getFormCellWiUndefinedCells( true );

		$this->view->assign( 'area', $area );
		$this->view->assign( 'ticketTitle', $ticketTitle );
		$this->view->assign( 'cells', $this->_getAreaCells( $area ) );
		$this->view->assign( 'currentModeLabel', $currentModeLabel );
		$this->view->assign( 'formCellModeOptions', $formCellModeOptions );
		$this->view->assign( 'formCellModeSelected', $formCellModeSelected );
		$this->view->assign( 'formCellTicketOptions', $formCellTicketOptions );
		$this->view->assign( 'formCellTicketSelected', $formCellTicketSelected );
		$this->view->assign( 'formCellWiBgimageChecked', $formCellWiBgimageChecked );
		$this->view->assign( 'formCellWiCategoriesChecked', $formCellWiCategoriesChecked );
		$this->view->assign( 'formCellWiGroupsChecked', $formCellWiGroupsChecked );
		$this->view->assign( 'formCellWiTicketsChecked', $formCellWiTicketsChecked );
		$this->view->assign( 'formCellWiUndefinedCellsChecked', $formCellWiUndefinedCellsChecked );
		$this->view->assign( 'grid-template-columns', AreaModuleUtility::getAutoElementsForGridCSS( $area ) );
		$this->view->assign( 'size', '15' );
	}

	/**
	 * initializeAction: is executed before any Action in the Controller.
	 * 
	 * @return void
	 * @version 0.0.3
	 * @since 0.0.3
	 * @internal #t3728
	 */
	public function initializeAction()
	{
		SessionModuleUtility::setSession();
	}

	/**
	 * action updatecell
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return void
	 * @version 0.0.6
	 * @since 0.0.1
	 */
	public function updatecellAction( \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		switch( $this->_cellRangeIsEditable( $cell ) )
		{
			case(true):
				$this->_setTicketForCells( $cell );
				break;
			case(false):
			default:
				$this->_promptCellRangeIsNotEmpty();
				SessionModuleUtility::removeValue( 'formerCellUid' );
				break;
		}
		$area = $this->request->getArgument( 'area' );
		$arguments = [
				'area' => $area
		];
		$this->redirect( 'enable', NULL, NULL, $arguments );
	}

}
