<?php

namespace Netzmacher\Tile\Domain\Model;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * Area
 */
class Area extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

	const TABLE_NAME = 'tx_tile_domain_model_area';

	/**
	 * image
	 * 
	 * @var \TYPO3\CMS\Extbase\Domain\Model\FileReference
	 * @internal http://keinerweiss.de/755-typo3-fal-in-einer-eigenen-extbasefluid-extension-einsetzen.html
	 */
	protected $image = 0;

	/**
	 * lenx
	 * 
	 * @var int
	 * @validate NotEmpty
	 */
	protected $lenx = 0;

	/**
	 * leny
	 * 
	 * @var string
	 * @validate NotEmpty
	 */
	protected $leny = '';

	/**
	 * pid
	 * 
	 * @var int
	 */
	protected $pid = null;

	/**
	 * title
	 * 
	 * @var string
	 * @validate NotEmpty
	 */
	protected $title = '';

	/**
	 * type
	 * 
	 * @var string
	 */
	protected $type = '';

	/**
	 * txTileCell
	 * 
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Netzmacher\Tile\Domain\Model\Cell>
	 * @cascade remove
	 * @lazy
	 */
	protected $txTileCell = null;

	/**
	 * Returns the image
	 * 
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\FileReference> $image
	 * @internal http://keinerweiss.de/755-typo3-fal-in-einer-eigenen-extbasefluid-extension-einsetzen.html
	 */
	public function getImage()
	{
		return $this->image;
	}

	/**
	 * Sets the image
	 * 
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\TYPO3\CMS\Extbase\Domain\Model\FileReference> $image
	 * @return void
	 * @internal http://keinerweiss.de/755-typo3-fal-in-einer-eigenen-extbasefluid-extension-einsetzen.html
	 */
	public function setImage( $image )
	{
		$this->image = $image;
	}

	/**
	 * Returns the lenx
	 * 
	 * @return int $lenx
	 */
	public function getLenx()
	{
		return $this->lenx;
	}

	/**
	 * Sets the lenx
	 * 
	 * @param int $lenx
	 * @return void
	 */
	public function setLenx( $lenx )
	{
		$this->lenx = $lenx;
	}

	/**
	 * Returns the leny
	 * 
	 * @return string $leny
	 */
	public function getLeny()
	{
		return $this->leny;
	}

	/**
	 * Sets the leny
	 * 
	 * @param string $leny
	 * @return void
	 */
	public function setLeny( $leny )
	{
		$this->leny = $leny;
	}

	/**
	 * Returns the pid
	 * 
	 * @return string $pid
	 */
	public function getPid()
	{
		return $this->pid;
	}

	/**
	 * Returns the title
	 * 
	 * @return string $title
	 */
	public function getTitle()
	{
		return $this->title;
	}

	/**
	 * Sets the title
	 * 
	 * @param string $title
	 * @return void
	 */
	public function setTitle( $title )
	{
		$this->title = $title;
	}

	/**
	 * Returns the type
	 * 
	 * @return int type
	 */
	public function getType()
	{
		return $this->type;
	}

	/**
	 * Sets the type
	 * 
	 * @param string $type
	 * @return void
	 */
	public function setType( $type )
	{
		$this->type = $type;
	}

	/**
	 * __construct
	 */
	public function __construct()
	{

		//Do not remove the next line: It would break the functionality
		$this->initStorageObjects();
	}

	/**
	 * Initializes all ObjectStorage properties
	 * 
	 * @return void
	 */
	protected function initStorageObjects()
	{
		$this->txTileCell = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
	}

	/**
	 * Adds a Cell
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $txTileCell
	 * @return void
	 */
	public function addTxTileCell( \Netzmacher\Tile\Domain\Model\Cell $txTileCell )
	{
		$this->txTileCell->attach( $txTileCell );
	}

	/**
	 * Removes a Cell
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Cell $txTileCellToRemove The Cell to be removed
	 * @return void
	 */
	public function removeTxTileCell( \Netzmacher\Tile\Domain\Model\Cell $txTileCellToRemove )
	{
		$this->txTileCell->detach( $txTileCellToRemove );
	}

	/**
	 * Returns the txTileCell
	 * 
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Netzmacher\Tile\Domain\Model\Cell> txTileCell
	 */
	public function getTxTileCell()
	{
		return $this->txTileCell;
	}

	/**
	 * Sets the txTileCell
	 * 
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\Netzmacher\Tile\Domain\Model\Cell> $txTileCell
	 * @return void
	 */
	public function setTxTileCell( \TYPO3\CMS\Extbase\Persistence\ObjectStorage $txTileCell )
	{
		$this->txTileCell = $txTileCell;
	}

}
