<?php

namespace Netzmacher\Tile\Domain\Model;

use Netzmacher\Tile\Utility\Module\SessionModuleUtility;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * Cell
 */
class Cell extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

	const TABLE_NAME = 'tx_tile_domain_model_cell';

	/**
	 * groupcss
	 * 
	 * @var string
	 */
	protected $groupcss = '';

	/**
	 * title
	 * 
	 * @var string
	 * @validate NotEmpty
	 */
	protected $title = '';

	/**
	 * txTileArea
	 * 
	 * @var \Netzmacher\Tile\Domain\Model\Area
	 */
	protected $txTileArea = null;

	/**
	 * txTileCategory
	 * 
	 * @var int
	 */
	protected $txTileCategory = null;

	/**
	 * txTileGroup
	 * 
	 * @var \Netzmacher\Tile\Domain\Model\Group
	 */
	protected $txTileGroup = null;

	/**
	 * txTileProduct
	 * 
	 * @var \Netzmacher\Tile\Domain\Model\Product
	 */
	protected $txTileProduct = null;

	/**
	 * txTileTicket
	 * 
	 * @var int
	 */
	protected $txTileTicket = null;

	/**
	 * x
	 * 
	 * @var int
	 * @validate NotEmpty
	 */
	protected $x = 0;

	/**
	 * y
	 * 
	 * @var int
	 * @validate NotEmpty
	 */
	protected $y = 0;

	/**
	 * Sets the groupcss
	 * 
	 * @return string
	 */
	private function _groupCssCell(): string
	{
		return 'north east south west';
	}

	/**
	 * Sets the groupcss
	 * 
	 * @param array $range: maxX, maxY, minX, minY
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return string
	 */
	private function _groupCssGroup( $range, \Netzmacher\Tile\Domain\Model\Cell $cell )
	{
		$cssClass = null;
		$devider = null;

		if( $cell->getX() == $range[ 'minX' ] )
		{
			$cssClass = $cssClass . $devider . 'west';
			$devider = ' ';
		}
		if( $cell->getX() == $range[ 'maxX' ] )
		{
			$cssClass = $cssClass . $devider . 'east';
			$devider = ' ';
		}
		if( $cell->getY() == $range[ 'minY' ] )
		{
			$cssClass = $cssClass . $devider . 'north';
			$devider = ' ';
		}
		if( $cell->getY() == $range[ 'maxY' ] )
		{
			$cssClass = $cssClass . $devider . 'south';
			$devider = ' ';
		}

		return $cssClass;
	}

	/**
	 * Returns the groupcss
	 * 
	 * @return string $groupcss
	 */
	public function getGroupcss()
	{
		return $this->groupcss;
	}

	/**
	 * Sets the groupcss
	 * 
	 * @param string $mode: cell | group
	 * @param array $range: maxX, maxY, minX, minY
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @return string
	 */
	public function setGroupcss( $mode, $range = array(), \Netzmacher\Tile\Domain\Model\Cell $cell = null )
	{
		if( SessionModuleUtility::getFormCellTicket() == 0 )
		{
			$this->groupcss = '';
			return;
		}

		switch( $mode )
		{
			case('cell'):
				$groupcss = $this->_groupCssCell();
				break;
			case('group'):
				$groupcss = $this->_groupCssGroup( $range, $cell );
				break;
			default:
				var_dump( __METHOD__, __LINE__, 'Error in switch! Unexpected value "' . $mode . '"' );
				die();
		}
		$this->groupcss = $groupcss;
	}

	/**
	 * Returns the title
	 * 
	 * @return string $title
	 */
	public function getTitle()
	{
		return $this->title;
	}

	/**
	 * Sets the title
	 * 
	 * @param string $title
	 * @return void
	 */
	public function setTitle( $title )
	{
		$this->title = $title;
	}

	/**
	 * Returns the x
	 * 
	 * @return int $x
	 */
	public function getX()
	{
		return $this->x;
	}

	/**
	 * Sets the x
	 * 
	 * @param int $x
	 * @return void
	 */
	public function setX( $x )
	{
		$this->x = $x;
	}

	/**
	 * Returns the y
	 * 
	 * @return int $y
	 */
	public function getY()
	{
		return $this->y;
	}

	/**
	 * Sets the y
	 * 
	 * @param int $y
	 * @return void
	 */
	public function setY( $y )
	{
		$this->y = $y;
	}

	/**
	 * Returns the txTileArea
	 * 
	 * @return \Netzmacher\Tile\Domain\Model\Area txTileArea
	 */
	public function getTxTileArea()
	{
		return $this->txTileArea;
	}

	/**
	 * Sets the txTileArea
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Area $txTileArea
	 * @return void
	 */
	public function setTxTileArea( \Netzmacher\Tile\Domain\Model\Area $txTileArea )
	{
		$this->txTileArea = $txTileArea;
	}

	/**
	 * Returns the txTileCategory
	 * 
	 * @return int
	 */
	public function getTxTileCategory()
	{
		return $this->txTileCategory;
	}

	/**
	 * Sets the txTileCategory
	 * 
	 * @param int
	 * @return void
	 */
	public function setTxTileCategory( $categoryUid )
	{
		$this->txTileCategory = $categoryUid;
	}

	/**
	 * Returns the txTileGroup
	 * 
	 * @return \Netzmacher\Tile\Domain\Model\Group $txTileGroup
	 */
	public function getTxTileGroup()
	{
		return $this->txTileGroup;
	}

	/**
	 * Removes a Group
	 * 
	 * @return void
	 */
	public function removeTxTileGroup()
	{
		$this->txTileGroup = null;
	}

	/**
	 * Sets the txTileGroup
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Group $txTileGroup
	 * @return void
	 */
	public function setTxTileGroup( \Netzmacher\Tile\Domain\Model\Group $txTileGroup )
	{
		$this->txTileGroup = $txTileGroup;
	}

	/**
	 * Returns the txTileProduct
	 * 
	 * @return \Netzmacher\Tile\Domain\Model\Product $txTileProduct
	 */
	public function getTxTileProduct()
	{
		return $this->txTileProduct;
	}

	/**
	 * Removes a Product
	 * 
	 * @return void
	 */
	public function removeTxTileProduct()
	{
		$this->txTileProduct = null;
	}

	/**
	 * Sets the txTileProduct
	 * 
	 * @param \Netzmacher\Tile\Domain\Model\Product $txTileProduct
	 * @return void
	 */
	public function setTxTileProduct( \Netzmacher\Tile\Domain\Model\Product $txTileProduct )
	{
		$this->txTileProduct = $txTileProduct;
	}

	/**
	 * Returns the txTileTicket
	 * 
	 * @return int
	 */
	public function getTxTileTicket()
	{
		return $this->txTileTicket;
	}

	/**
	 * Sets the txTileTicket
	 * 
	 * @param int
	 * @return int
	 */
	public function setTxTileTicket( $ticketUid )
	{
		$this->txTileTicket = $ticketUid;
	}

}
