<?php

namespace Netzmacher\Tile\Domain\Repository;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * The repository for Cells
 */
class CellRepository extends \TYPO3\CMS\Extbase\Persistence\Repository
{

	/**
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return int
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3835
	 */
	public function countCellsByArea( \Netzmacher\Tile\Domain\Model\Area $area ): int
	{
		$rows = $this->getCellsByArea( $area );
		$sum = count( $rows );

		return $sum;
	}

	/**
	 * @param \Netzmacher\Tile\Domain\Model\Area $area
	 * @return array
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3835
	 */
	public function getCellsByArea( \Netzmacher\Tile\Domain\Model\Area $area ): array
	{
		$rows = [];
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tx_tile_domain_model_cell' )->createQueryBuilder();
		$queryBuilder
						->select( '*' )
						->from( 'tx_tile_domain_model_cell' )
						->where(
										$queryBuilder->expr()->eq( 'tx_tile_area', $queryBuilder->createNamedParameter( $area->getUid() ) )
						)
		;

		$result = $queryBuilder->execute();

		while( $row = $result->fetch() )
		{
			$rows[] = $row;
		}

		return $rows;
	}

	/**
	 * @param \Netzmacher\Tile\Domain\Model\Group $group
	 * @return array
	 * @version 0.0.6
	 * @since 0.0.5
	 * @internal #t3749
	 */
	public function getCellsByGroup( \Netzmacher\Tile\Domain\Model\Group $group ): array
	{
		$rows = [];
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tx_tile_domain_model_cell' )->createQueryBuilder();
		$queryBuilder
						->select( 'uid' )
						->from( 'tx_tile_domain_model_cell' )
						->where(
										$queryBuilder->expr()->eq( 'tx_tile_group', $queryBuilder->createNamedParameter( $group->getUid() ) )
						)
		;

		$result = $queryBuilder->execute();

		while( $row = $result->fetch() )
		{
			$rows[] = $row[ 'uid' ];
		}

		return $rows;
	}

	/**
	 * @param \Netzmacher\Tile\Domain\Model\Cell $cell
	 * @param array
	 * @return array
	 * @version 0.0.6
	 * @since 0.0.5
	 * @internal #t3749
	 */
	public function getCellsByXyRange( \Netzmacher\Tile\Domain\Model\Cell $cell, $xyRange ): array
	{
//		var_dump( __METHOD__, __LINE__, $cell->getTxTileArea()->getTitle() );
//		die();
		$rows = [];
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tx_tile_domain_model_cell' )->createQueryBuilder();
		$queryBuilder
						->select( 'uid' )
						->from( 'tx_tile_domain_model_cell' )
						->where(
										$queryBuilder->expr()->gte( 'x', $queryBuilder->createNamedParameter( $xyRange[ 'minX' ] ) )
										, $queryBuilder->expr()->lte( 'x', $queryBuilder->createNamedParameter( $xyRange[ 'maxX' ] ) )
										, $queryBuilder->expr()->gte( 'y', $queryBuilder->createNamedParameter( $xyRange[ 'minY' ] ) )
										, $queryBuilder->expr()->lte( 'y', $queryBuilder->createNamedParameter( $xyRange[ 'maxY' ] ) )
										, $queryBuilder->expr()->eq( 'tx_tile_area', $queryBuilder->createNamedParameter( $cell->getTxTileArea()->getUid() ) )
						)
		;

		$result = $queryBuilder->execute();

		while( $row = $result->fetch() )
		{
			$rows[] = $row[ 'uid' ];
		}

		return $rows;
	}

}
