<?php

namespace Netzmacher\Tile\Flexform;

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Database\ConnectionPool;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2019 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * AbstractController for the xBlog xtension
 *
 * @package TYPO3
 * @subpackage tile
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 0.0.8
 * @since 0.0.7
 */
class ItemsprocFlexform
{

	/**
	 * AreasCloned( ) : 
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	public function AreasCloned( &$params )
	{

		$rows = $this->_getAreasCloned();

		if( empty( $rows ) )
		{
			$params = $this->_promptCreateAreaCloneFirst( $params );
//			var_dump( __METHOD__, __LINE__, $rows, $params );
//			die();
			return $params;
		}

		$params[ 'items' ][] = [ '', '', '' ];

		foreach( ( array ) $rows as $row )
		{
			// if not clone
			//		continue;
			if( $row[ 'type' ] != 1 )
			{
				continue;
			}
			$params[ 'items' ][] = [
					$row[ 'title' ]
					, $row[ 'uid' ]
					, 'EXT:tile/Resources/Public/Icons/ext-tile-area-clone.svg'
			];
		}

//		var_dump( __METHOD__, __LINE__, $rows, $params );
//		die();

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * Caddy( ) : 
	 *
	 * @return array	item array for a flexform
	 * @access public
	 * @version 0.0.8
	 * @since 0.0.8
	 */
	public function Caddy( &$params )
	{

		$rows = $this->_getCaddies();

		if( empty( $rows ) )
		{
			$params = $this->_promptCreateCaddyFirst( $params );
//			var_dump( __METHOD__, __LINE__, $rows, $params );
//			die();
			return $params;
		}

		$params[ 'items' ][] = [ '', '', '' ];

		foreach( ( array ) $rows as $row )
		{
			$params[ 'items' ][] = [
					$row[ 'header' ] . ' (on page #' . $row[ 'pid' ] . ')'
					, $row[ 'uid' ]
					, 'EXT:caddy/ext_icon.svg'
			];
		}

//		var_dump( __METHOD__, __LINE__, $rows, $params );
//		die();

		array_multisort( array_column( $params[ 'items' ], 0 ), SORT_ASC, $params[ 'items' ] );

		return $params;
	}

	/**
	 * _getAreasCloned( ) : 
	 *
	 * @return array	rows
	 * @access public
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _getAreasCloned()
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tx_tile_domain_model_cell' )->createQueryBuilder();
		$queryBuilder
						->select( '*' )
						->from( 'tx_tile_domain_model_area' )
						->where(
										$queryBuilder->expr()->eq( 'type', $queryBuilder->createNamedParameter( '1' ) )
						)
		;

		$result = $queryBuilder->execute();

		while( $row = $result->fetch() )
		{
			$rows[] = $row;
		}

		return $rows;
	}

	/**
	 * _getCaddies( ) : 
	 *
	 * @return array	rows
	 * @access public
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _getCaddies()
	{
		$queryBuilder = GeneralUtility::makeInstance( ConnectionPool::class )->getConnectionForTable( 'tt_content' )->createQueryBuilder();
		$queryBuilder
						->select( '*' )
						->from( 'tt_content' )
						->where(
										$queryBuilder->expr()->eq( 'CType', $queryBuilder->createNamedParameter( 'list' ) )
										, $queryBuilder->expr()->eq( 'list_type', $queryBuilder->createNamedParameter( 'caddy_pi1' ) )
						)
		;

		$result = $queryBuilder->execute();

		while( $row = $result->fetch() )
		{
			$rows[] = $row;
		}

		return $rows;
	}

	/**
	 * _promptCreateAreaCloneFirst( ) : 
	 *
	 * @return array	item array for a flexform
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _promptCreateAreaCloneFirst( &$params )
	{

		$params[ 'items' ][] = [
				'You have to create a live version of a site plan first.'
				, null
				, ''
		];

		return $params;
	}

	/**
	 * _promptCreateCaddyFirst( ) : 
	 *
	 * @return array	item array for a flexform
	 * @version 0.0.8
	 * @since 0.0.8
	 */
	private function _promptCreatecaddyFirst( &$params )
	{

		$params[ 'items' ][] = [
				'You have to create a Caddy plugin first.'
				, null
				, ''
		];

		return $params;
	}

}
