<?php

namespace Netzmacher\Tile\Utility\Caddy;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * CaddySessionUtility
 * 
 * @version 0.0.8
 * @since 0.0.8
 * @internal #t3869
 */
class CaddySessionUtility
{

	/**
	 * Pid of the Caddy plugin
	 * 
	 * @var integer
	 */
	private $_pid = null;

	/**
	 * Caddy extension key
	 * 
	 * @var string
	 */
	private $_extKey = 'caddy';

	/**
	 * __construct()
	 * 
	 * @param integer pid
	 * 
	 * @return void
	 * @throws \RuntimeException
	 * @version 0.0.8
	 * @since 0.0.8
	 */
	public function __construct( $pid )
	{
		if( !empty( $pid ) )
		{
			$this->_pid = $pid;
			return;
		}
		$prompt = 'ERROR @ ' . __METHOD__ . ' (#' . __LINE__ . ')';
		die( $prompt );
	}

	/**
	 * GetSessionData
	 * 
	 * @return mixed
	 * @version 0.0.8
	 * @since 0.0.8
	 */
	public function GetSessionData()
	{
		return $this->_getSessionData();
	}

	/**
	 * GetSessionElement: get an element from the session data
	 * 
	 * @param string $element
	 * @param string $default: default value
	 * @return mixed
	 * @version 0.0.8
	 * @since 0.0.8
	 */
	public function GetSessionElement( $element, $default = null )
	{
		$sessionData = $this->_getSessionData();

		if( isset( $sessionData[ $element ] ) )
		{
			$value = $sessionData[ $element ];
			return $value;
		}
		$value = $sessionData[ $element ];
		return $default;
	}

	/**
	 * _getSessionData
	 * 
	 * @return array
	 * @version 0.0.8
	 * @since 0.0.8
	 */
	private function _getSessionData()
	{

		$space = $this->_getSessionSpace();
		$sessionData = $GLOBALS[ 'TSFE' ]->fe_user->getKey( $space, $this->_extKey . '_' . $this->_pid );
//var_dump( __METHOD__, __LINE__, $space, $this->_extKey . '_' . $this->_pid, $sessionData );
		return $sessionData;
	}

	/**
	 * _getSessionSpace( ): Get the name of the session data space.
	 *                        The name is user, if a frontend user is logged in.
	 *                        The name is ses, if any frontend user isn't logged in.
	 *
	 * @return	string		user || ses
	 * @version 0.5.2
	 * @since 0.5.2
	 */
	private function _getSessionSpace()
	{
		switch( $GLOBALS[ 'TSFE' ]->loginUser )
		{
			case( true ):
				return 'user';
			default:
				return 'ses';
		}
	}

}
