<?php

namespace Netzmacher\Tile\Utility\Module;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * SessionModuleUtility
 * 
 * @version 0.0.6
 * @since 0.0.3
 * @internal #t3728
 */
class SessionModuleUtility
{

	/**
	 * _getSession: set session data
	 * 
	 * @return mixed
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	static private function _getSession()
	{
		global $BE_USER;

		$sessionData = $BE_USER->getModuleData( 'web_tilemd1', 'ses' );
		return $sessionData;
	}

	/**
	 * _getSessionElement: get an element from the session data
	 * 
	 * @param string $element
	 * @param string $default: default value
	 * @return mixed
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	static private function _getSessionElement( $element, $default = null )
	{
		$sessionData = self::_getSession();

		if( isset( $sessionData[ $element ] ) )
		{
			$value = $sessionData[ $element ];
			return $value;
		}
		$value = $sessionData[ $element ];
		return $default;
	}

	/**
	 * _setSessionElement: set session data
	 * 
	 * @param string	$key
	 * @param	string	$value
	 *  
	 * @return void
	 * @version 0.0.5
	 * @since 0.0.3
	 */
	static public function _setSessionElement( $key, $value )
	{
		global $BE_USER;

		$sessionData = self::_getSession();

		if( $value === NULL )
		{
			unset( $sessionData[ $key ] );
			$BE_USER->pushModuleData( 'web_tilemd1', $sessionData );
			return;
		}
		$sessionData[ $key ] = $value;

		$BE_USER->pushModuleData( 'web_tilemd1', $sessionData );
	}

	/**
	 * getFormCellCategory: get tx_tile_category from the session data
	 * 
	 * @return string
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	static public function getFormCellCategory()
	{
		$value = self::_getSessionElement( 'formCellCategory' );
		return $value;
	}

	/**
	 * getFormCellMode: get moede from the session data
	 * 
	 * @return string
	 * @version 0.0.5
	 * @since 0.0.5
	 */
	static public function getFormCellMode()
	{
		$value = self::_getSessionElement( 'formCellMode' );
		return $value;
	}

	/**
	 * getFormCellTicket: get tx_tile_ticket from the session data
	 * 
	 * @return string
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	static public function getFormCellTicket()
	{
		$value = self::_getSessionElement( 'formCellTicket' );
		return $value;
	}

	/**
	 * getFormCellWiBgimage
	 * 
	 * @param string $default: default value
	 * @return string
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	static public function getFormCellWiBgimage( $default )
	{
		$value = self::_getSessionElement( 'formCellWiBgimage', $default );
		return $value;
	}

	/**
	 * getFormCellWiCategories
	 * 
	 * @param string $default: default value
	 * @return string
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	static public function getFormCellWiCategories( $default )
	{
		$value = self::_getSessionElement( 'formCellWiCategories', $default );
		return $value;
	}

	/**
	 * getFormCellWiGroups
	 * 
	 * @param string $default: default value
	 * @return string
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	static public function getFormCellWiGroups( $default )
	{
		$value = self::_getSessionElement( 'formCellWiGroups', $default );
		return $value;
	}

	/**
	 * getFormCellWiTickets
	 * 
	 * @param string $default: default value
	 * @return string
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	static public function getFormCellWiTickets( $default )
	{
		$value = self::_getSessionElement( 'formCellWiTickets', $default );
		return $value;
	}

	/**
	 * getFormCellWiUndefinedCells
	 * 
	 * @param string $default: default value
	 * @return string
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3812
	 */
	static public function getFormCellWiUndefinedCells( $default )
	{
		$value = self::_getSessionElement( 'formCellWiUndefinedCells', $default );
		return $value;
	}

	/**
	 * getFormShopDestpid: 
	 * 
	 * @return string
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3841
	 */
	static public function getFormShopDestpid()
	{
		$value = self::_getSessionElement( 'formShopDestpid' );
		return $value;
	}

	/**
	 * getFormShopTitle: 
	 * 
	 * @return string
	 * @version 0.0.6
	 * @since 0.0.6
	 * @internal #t3863
	 */
	static public function getFormShopTitle()
	{
		$value = self::_getSessionElement( 'formShopTitle' );
		return $value;
	}
	
	/**
	 * getValue()
	 * 
	 * @return string
	 * @version 0.0.3
	 * @since 0.0.3
	 */
	static public function getValue( $key )
	{
		$value = self::_getSessionElement( $key );
		return $value;
	}

	/**
	 * removeValue()
	 * 
	 * @param string key
	 * @param string value
	 * 
	 * @return string
	 * @version 0.0.5
	 * @since 0.0.5
	 */
	static public function removeValue( $key )
	{
		self::_setSessionElement( $key, NULL );
	}

	/**
	 * setValue()
	 * 
	 * @param string key
	 * @param string value
	 * 
	 * @return string
	 * @version 0.0.5
	 * @since 0.0.5
	 */
	static public function setValue( $key, $value )
	{
		self::_setSessionElement( $key, $value );
	}

	/**
	 * setSession: set session data
	 * 
	 * @return void
	 * @version 0.0.5
	 * @since 0.0.3
	 */
	static public function setSession()
	{
		global $BE_USER;

		$moduleGPData = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( 'tx_tile_web_tilemd1' );
		if( empty( $moduleGPData ) )
		{
			return;
		}

		$sessionData = self::_getSession();

		foreach( $moduleGPData as $key => $value )
		{
			if( $value === NULL )
			{
				unset( $sessionData[ $key ] );
				continue;
			}
			$sessionData[ $key ] = $value;
		}
		$BE_USER->pushModuleData( 'web_tilemd1', $sessionData );
		return;
	}

}
