<?php

namespace Netzmacher\Tile\Utility;

/* * *
 *
 * This file is part of the "Book a tile!" Extension for TYPO3 CMS.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 *  (c) 2019 - Dirk Wildt (Die Netzmacher) <http://wildt@die-netzmacher.de>, Die Netzmacher
 *
 * * */

/**
 * SessionUtility
 * 
 * @version 0.0.7
 * @since 0.0.7
 * @internal #t3866
 */
class SessionUtility
{

	/**
	 * Data of the current object / plugin (tt_content)
	 * 
	 * @var array
	 */
	private $_cData = [];

	/**
	 * _getSessionData
	 * 
	 * @return array
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _getSessionData()
	{
		$plugin = 'tx_' . $this->_cData[ 'list_type' ];
		$space = $this->_getSessionSpace();
		$sessionData = $GLOBALS[ 'TSFE' ]->fe_user->getKey( $space, $plugin );
		return $sessionData;
	}

	/**
	 * _getSessionSpace( ): Get the name of the session data space.
	 *                        The name is user, if a frontend user is logged in.
	 *                        The name is ses, if any frontend user isn't logged in.
	 *
	 * @return	string		user || ses
	 * @version 0.5.2
	 * @since 0.5.2
	 */
	private function _getSessionSpace()
	{
		switch( $GLOBALS[ 'TSFE' ]->loginUser )
		{
			case( true ):
				return 'user';
			default:
				return 'ses';
		}
	}

	/**
	 * setSessionContinue
	 * 
	 * @param array $arrSessionData
	 * @param string $key
	 * @param string $value
	 * @return boolean
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _setSessionContinue( &$arrSessionData, $key, $value ): bool
	{
		$uid = $this->_cData[ 'uid' ];
		$pos = strpos( $key, '__' );
		switch( true )
		{
			case($pos === ( int ) 0):
				return true;
			case($value === NULL):
				unset( $arrSessionData[ $uid ][ $key ] );
				return true;
			default:
				return false;
		}
	}

	/**
	 * _setSessionData
	 * 
	 * @param array $pluginData: data of the current form
	 * @param string $plugin: f.e. tx_tile_pi1
	 * @param int $uid: uid of the current plugin
	 * @param array $sessionData: current session data
	 * @return array
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	private function _setSessionData( $pluginData, $sessionData )
	{
		$uid = $this->_cData[ 'uid' ];
		$plugin = 'tx_' . $this->_cData[ 'list_type' ];
		foreach( $pluginData as $key => $value )
		{
			if( $this->_setSessionContinue( $sessionData, $key, $value ) )
			{
				continue;
			}
			$sessionData[ $uid ][ $key ] = $value;
		}

		$space = $this->_getSessionSpace();
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( $space, $plugin, $sessionData );
		$GLOBALS[ "TSFE" ]->storeSessionData();

		return $sessionData;
	}

//	/**
//	 * getFormShopTitle: 
//	 * 
//	 * @return string
//	 * @version 0.0.7
//	 * @since 0.0.7
//	 * @internal #t3863
//	 */
//	public function getFormShopTitle()
//	{
//		$value = $this->_getSessionElement( 'formShopTitle' );
//		return $value;
//	}
//
	/**
	 * getSessionElement: get an element from the session data
	 * 
	 * @param string $element
	 * @param string $default: default value
	 * @return mixed
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	public function getSessionElement( $element, $default = null )
	{
		$uid = $this->_cData[ 'uid' ];

		$sessionData = $this->_getSessionData();

		if( isset( $sessionData[ $uid ][ $element ] ) )
		{
			$value = $sessionData[ $uid ][ $element ];
			return $value;
		}
		$value = $sessionData[ $uid ][ $element ];
		return $default;
	}

	/**
	 * setCData()
	 * 
	 * @param string key
	 * @param string value
	 * 
	 * @return string
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	public function setCData( $cData )
	{
		$this->_cData = $cData;
	}
//
//	/**
//	 * setValue()
//	 * 
//	 * @param string key
//	 * @param string value
//	 * 
//	 * @return string
//	 * @version 0.0.7
//	 * @since 0.0.7
//	 */
//	public function setValue( $key, $value )
//	{
//		$this->_setSessionElement( $key, $value );
//	}

	/**
	 * setSession: set session data
	 * 
	 * @return void
	 * @version 0.0.7
	 * @since 0.0.7
	 */
	public function setSession()
	{
		$plugin = 'tx_' . $this->_cData[ 'list_type' ];
		$pluginData = \TYPO3\CMS\Core\Utility\GeneralUtility::_GP( $plugin );
		//var_dump( __METHOD__, __LINE__, $plugin, $pluginData );
		// Do not remove session data
		if( empty( $pluginData ) )
		{
			return;
		}

		$sessionData = $this->_getSessionData();
		$sessionData = $this->_setSessionData( $pluginData, $sessionData );

		//var_dump( __METHOD__, __LINE__, $sessionData );

		return;
	}

}
