<?php

//declare(strict_types=1);

namespace Netzmacher\Tile\ViewHelpers\Get;

use Netzmacher\Tile\Domain\Repository\CategoryRepository;
use Netzmacher\Tile\Domain\Repository\ContentRepository;
use Netzmacher\Tile\Domain\Repository\TicketRepository;
use Netzmacher\Tile\Utility\Caddy\CaddySessionUtility;
use Netzmacher\Tile\Utility\ObjectUtility;
use Netzmacher\Tile\Utility\SessionUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class CellClassViewHelper
 * 
 * @version 0.0.8
 * @since 0.0.8
 * @internal #t3816
 */
class CellClassViewHelper extends AbstractFlexformViewHelper
{

	/**
	 * Data of the current content object
	 * 
	 * @var array
	 */
	private $_cData = null;

	/**
	 * Current cell
	 * 
	 * @var \Netzmacher\Tile\Domain\Model\Cell
	 */
	private $_cell = null;

	/**
	 * Category Repository
	 * 
	 * @var Netzmacher\Tile\Domain\Repository\CategoryRepository
	 */
	private $_categoryRepository = null;

	/**
	 * Prompts in context with  static varibales. For development only
	 * 
	 * @var boolean
	 */
	private $_promptStatic = false;

	/**
	 * Session class
	 * 
	 * @var Netzmacher\Tile\Utility\SessionUtility
	 */
	private $_sessionUtility = null;

	/**
	 * Ticket Repository
	 * 
	 * @var Netzmacher\Tile\Domain\Repository\TicketRepository
	 */
	private $_ticketRepository = null;

	/**
	 * @param string $class
	 * @param string $additionalClass
	 * @return mixed
	 */
	private function _classConcatenator( $class, $additionalClass )
	{
		switch( true )
		{
			case( empty( $additionalClass )):
				return $class;
			case( empty( $class ) ):
				return $additionalClass;
			default:
				return $class . ' ' . $additionalClass;
		}
	}

	/**
	 * @param string $formProperty
	 * @param boolean $default
	 * @return mixed
	 */
	private function _dontDisplay( $formProperty, $default = true )
	{
		static $display = null;

		if( isset( $display[ $formProperty ] ) )
		{
			return $display[ $formProperty ];
		}

		$display[ $formProperty ] = $this->_sessionUtility->getSessionElement( $formProperty, $default );

		$prompt = '[' . $formProperty . '] This prompt should display once only! Check of static variables:';
		$this->_promptStatic( __METHOD__, __LINE__, $prompt );
		if( empty( $display[ $formProperty ] ) )
		{
			$display[ $formProperty ] = true;
			return $display[ $formProperty ];
		}

		$display[ $formProperty ] = false;
		return $display[ $formProperty ];
	}

	/**
	 * @param integer $uid
	 * @return \Netzmacher\Tile\Domain\Model\Category $category
	 */
	private function _getCategoryByUid( $uid ): \Netzmacher\Tile\Domain\Model\Category
	{
		static $categories = null;

		if( isset( $categories[ $uid ] ) )
		{
			return $categories[ $uid ];
		}

		$categories[ $uid ] = $this->_categoryRepository->findByUid( $uid );
		$prompt = '[#' . $uid . ':' . $categories[ $uid ]->getTitle() . '] This prompt should display once only! Check of static variables:';
		$this->_promptStatic( __METHOD__, __LINE__, $prompt );
		return $categories[ $uid ];
	}

	/**
	 * @param string $class
	 * @return mixed
	 */
	private function _getClassCategory( $class )
	{
		if( $this->_dontDisplay( 'formCellWiCategories' ) )
		{
			return $class;
		}

		$additionalClass = $this->_getClassCategoryOfCurrentCell();
		return $this->_classConcatenator( $class, $additionalClass );
	}

	/**
	 * @return mixed
	 */
	private function _getClassCategoryOfCurrentCell()
	{
		$this->_initCategoryRepository();

		$uid = ( int ) $this->_cell->getTxTileCategory();
		if( empty( $uid ) )
		{
			return;
		}

		$category = $this->_getCategoryByUid( $uid );
		if( empty( $category ) )
		{
			return;
		}

		$strClass = $this->_getCssClassByCategory( $category, $uid );
		return $strClass;
	}

	/**
	 * @param string $class
	 * @return mixed
	 */
	private function _getClassCategoryUndefined( $class )
	{
		if( !empty( $class ) )
		{
			return $class;
		}

		if( $this->_dontDisplay( 'formCellWiUndefinedCells' ) )
		{
			return;
		}

		return 'undefined';
	}

	/**
	 * @param string $class
	 * @return mixed
	 */
	private function _getClassGroup( $class )
	{
		if( $this->_dontDisplay( 'formCellWiGroups' ) )
		{
			return $class;
		}

		$additionalClass = $this->_cell->getGroupcss();
		return $this->_classConcatenator( $class, $additionalClass );
	}

	/**
	 * @param string $class
	 * @param string $status: caddy | default | selled
	 * @return mixed
	 */
	private function _getClassTickets( $class, $status = 'default' )
	{
		static $classTickets = null;

		if( $this->_dontDisplay( 'formCellWiTickets' ) )
		{
			return $class;
		}


		$this->_initTicketRepository();
		$uidTicket = $this->_cell->getTxTileTicket();
		$ticket = $this->_ticketRepository->findByUid( $uidTicket );
		if( empty( $ticket ) )
		{
			return $class;
		}


		$uidProductsInTheCaddy = $this->_getUidsFromCaddy();
		$uidProduct = $this->_cell->getTxTileProduct()->getUid();
//var_dump( __METHOD__, __LINE__, $uidProduct, ( array ) $uidProductsInTheCaddy );
		switch( true )
		{
			case(in_array( $uidProduct, ( array ) $uidProductsInTheCaddy )):
				$status = 'caddy';
				break;
			case($this->_cell->getTxTileProduct()->getStockquantity() >= 1):
				$status = 'default';
				break;
			case($this->_cell->getTxTileProduct()):
				$status = 'selled';
				break;
			default:
				$prompt = 'Undefined value in switch @ ' . __METHOD__ . ' (#' . __LINE__ . ')';
				die( $prompt );
		}

		//var_dump(__METHOD__, __LINE__, $this->_cell->getTxTileProduct()->getUid());
		if( isset( $classTickets[ $uidProduct ][ $status ] ) )
		{
			$additionalClass = $classTickets[ $uidProduct ][ $status ];
			return $this->_classConcatenator( $class, $additionalClass );
		}

		switch( $status )
		{
			case('caddy'):
				$css = $ticket->getTxTileCsscaddy();
				break;
			case('default'):
				$css = $ticket->getTxTileCssdefault();
				break;
			case('selled'):
				$css = $ticket->getTxTileCssselled();
				break;
			default:
				$prompt = 'Undefined value in switch @ ' . __METHOD__ . ' (#' . __LINE__ . ')';
				die( $prompt );
		}

		if( empty( $css ) )
		{
			$this->_getClassTicketsPromptDie( __METHOD__, __LINE__, $ticket );
		}

		$prompt = '[#' . $uidProduct . '][' . $status . '] This prompt should display once only! Check of static variables:';
		//var_dump( __METHOD__, __LINE__, $this->_cell->getTxTileGroup()->getUid(), $uidProduct, $prompt );
		$this->_promptStatic( __METHOD__, __LINE__, $prompt, false );

		$classTickets[ $uidProduct ][ $status ] = $css->getClass();
		$additionalClass = $classTickets[ $uidProduct ][ $status ];
		return $this->_classConcatenator( $class, $additionalClass );
	}

	/**
	 * @param string $method
	 * @param string $line
	 * @param string $ticket
	 * @return void
	 */
	private function _getClassTicketsPromptDie( $method, $line, $ticket )
	{
		$prompt = 'ERROR @ ' . $method . ' (#' . $line . ')<br />'
						. '<br />'
						. 'It seems, that a CSS class of the ticket with the uid #' . $ticket->getUid() . ' is empty.<br />'
						. 'Please check the ticket!'
		;
		die( $prompt );
	}

	/**
	 * @param \Netzmacher\Tile\Domain\Model\Category $category
	 * @param integer $uidCategory
	 * @return string
	 */
	private function _getCssClassByCategory( \Netzmacher\Tile\Domain\Model\Category $category, int $uidCategory ): string
	{
		static $classes = null;
		//static $cssRepository = null;

		if( isset( $classes[ $uidCategory ] ) )
		{
			return $classes[ $uidCategory ];
		}

		$css = $category->getTxTileCss();
		$classes[ $uidCategory ] = $css->getClass();

		$prompt = '[#' . $css->getUid() . ': ' . $classes[ $uidCategory ] . '] This prompt should display once only! Check of static variables:';
		$this->_promptStatic( __METHOD__, __LINE__, $prompt );
		return $classes[ $uidCategory ];
	}

	/**
	 * @return array
	 */
	private function _getUidsFromCaddy()
	{
		static $uids = null;

		if( $uids !== null )
		{
			return $uids;
		}

		$xmlFlexform = $this->_cData[ 'pi_flexform' ];
		$sheet = 'caddy';
		$field = 'settings.flexform.pi1.caddy.uid';
		$caddyUid = $this->_getFlexformValue( __METHOD__, __LINE__, $xmlFlexform, $sheet, $field );

		if( empty( $caddyUid ) )
		{
			$uids = false;
			return $uid;
		}

		$contentRepository = ObjectUtility::getObjectManager()->get( ContentRepository::class );

		$content = $contentRepository->findByUid( $caddyUid );
		if( empty( $content ) )
		{
			$uids = false;
			return $uid;
		}
		$caddyPid = $content->getPid();

//var_dump( __METHOD__, __LINE__,$caddyUid, $caddyPid );
		$CaddySessionUtility = GeneralUtility::makeInstance( CaddySessionUtility::class, $caddyPid );
		$sessionData = $CaddySessionUtility->GetSessionData();
//var_dump( __METHOD__, __LINE__,$sessionData );

		foreach( ( array ) $sessionData[ 'items' ] as $item )
		{
			$uids[] = $item[ 'uid' ];
		}

		$this->_promptStatic( __METHOD__, __LINE__ );
		return $uids;
	}

	/**
	 * Init the Category Repository once
	 * 
	 * @return void
	 */
	private function _initCategoryRepository()
	{
		static $categoryRepository = null;

		if( $categoryRepository !== null )
		{
			$this->_categoryRepository = $categoryRepository;
			return;
		}

		$categoryRepository = ObjectUtility::getObjectManager()->get( CategoryRepository::class );
		$this->_categoryRepository = $categoryRepository;

		$this->_promptStatic( __METHOD__, __LINE__ );
	}

	/**
	 * Init the Ticket Repository once
	 * 
	 * @return void
	 */
	private function _initTicketRepository()
	{
		static $ticketRepository = null;

		if( $ticketRepository !== null )
		{
			$this->_ticketRepository = $ticketRepository;
			return;
		}

		$ticketRepository = ObjectUtility::getObjectManager()->get( TicketRepository::class );
		$this->_ticketRepository = $ticketRepository;

		$this->_promptStatic( __METHOD__, __LINE__ );
	}

	/**
	 * Init the session class once
	 * 
	 * @return void
	 */
	private function _initSessionObject()
	{
		static $sessionUtility = null;

		if( $sessionUtility !== null )
		{
			$this->_sessionUtility = $sessionUtility;
			$this->_sessionUtility->setCData( $this->_cData );
			return;
		}

		$sessionUtility = GeneralUtility::makeInstance( SessionUtility::class );
		$this->_sessionUtility = $sessionUtility;
		$this->_sessionUtility->setCData( $this->_cData );
		$this->_promptStatic( __METHOD__, __LINE__ );
	}

	/**
	 * Prompt in context with static variables
	 * 
	 * @param string $method
	 * @param string $line
	 * @param string $prompt
	 * @param boolean $display
	 * 
	 * @return void
	 */
	private function _promptStatic( $method, $line, $prompt = 'This prompt should display once only! Check of static variables:', $display = false )
	{
		switch( true )
		{
			case($display):
				//follow the workflow
				break;
			case(!$this->_promptStatic ):
				return;
		}
		echo '[#' . $line . '] ' . $prompt . ' @ ' . $method . ' (#' . $line . ')<br />' . PHP_EOL;
	}

	/**
	 * @return void
	 */
	public function initializeArguments()
	{
		parent::initializeArguments();
		$this->registerArgument( 'cell', 'Netzmacher\Tile\Domain\Model\Cell', 'cell', false, null );
		$this->registerArgument( 'cData', 'array', 'cData', false, null );
	}

	/**
	 * @return mixed
	 */
	public function render()
	{
		$class = null;
		$cell = $this->arguments[ 'cell' ];
		$this->_cData = $this->arguments[ 'cData' ];

		if( empty( $cell ) )
		{
			return $class;
		}

		$this->_cell = $cell;
		$this->_initSessionObject();

		$class = $this->_getClassCategory( $class );
		$class = $this->_getClassCategoryUndefined( $class );
		$class = $this->_getClassGroup( $class );
		$class = $this->_getClassTickets( $class );

		return $class;
	}

}
