<?php

//declare(strict_types=1);

namespace Netzmacher\Tile\ViewHelpers\Get;

use Netzmacher\Tile\Domain\Repository\CssRepository;
use Netzmacher\Tile\Domain\Repository\CellRepository;
use Netzmacher\Tile\Utility\ObjectUtility;
use TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper;

/**
 * Class CssFieldViewHelper
 * 
 * @version 0.0.3
 * @since 0.0.3
 * @internal #t3731
 */
class CssFieldViewHelper extends AbstractViewHelper
{

	/**
	 * @return string
	 */
	private function _cssCell(): string
	{
		$default = $this->arguments[ 'default' ];
		$field = $this->arguments[ 'field' ];
		$uid = $this->arguments[ 'uidCell' ];

		switch( true )
		{
			case( empty( $uid ) ):
			case( empty( $field ) ):
				return $default;
			default :
				// follow the workflow
				break;
		}

		$cssCell = ObjectUtility::getObjectManager()->get( CellRepository::class );
		$cell = $cssCell->findByUid( $uid );

		if( empty( $cell ) )
		{
			return '';
		}

		switch( $field )
		{
			case('classAndGroupcss'):
			case('groupcss'):
				return $cell->getGroupcss();
			case('class'):
			case('color'):
			case('opacity'):
			case('title'):
				return '';
			default:
				var_dump( __METHOD__, __LINE__, 'Error in switch! Undefined field "' . $field . '"' );
				die();
		}
	}

	/**
	 * @return string
	 */
	private function _cssCategory(): string
	{
		$default = $this->arguments[ 'default' ];
		$field = $this->arguments[ 'field' ];
		$uid = $this->arguments[ 'uidCategory' ];

		switch( true )
		{
			case( empty( $uid ) ):
			case( empty( $field ) ):
				return $default;
			default :
				// follow the workflow
				break;
		}

		$cssCategory = ObjectUtility::getObjectManager()->get( CssRepository::class );
		$css = $cssCategory->findByUid( $uid );

		if( empty( $css ) )
		{
			return $default;
		}

		switch( $field )
		{
			case('class'):
			case('classAndGroupcss'):
				return $css->getClass();
			case('color'):
				return $css->getColor();
			case('groupcss'):
				return '';
			case('opacity'):
				return $css->getOpacity();
			case('title'):
				return $css->getTitle();
			default:
				var_dump( __METHOD__, __LINE__, 'Error in switch! Undefined field "' . $field . '"' );
				die();
		}
	}

	/**
	 * @return void
	 */
	public function initializeArguments()
	{
		parent::initializeArguments();
		$this->registerArgument( 'default', 'string', 'default', false, 'undefined' );
		$this->registerArgument( 'field', 'string', 'field', false, 0 );
		$this->registerArgument( 'uidCategory', 'int', 'uidCategory', false, 0 );
		$this->registerArgument( 'uidCell', 'int', 'uidCell', false, 0 );
	}

	/**
	 * @return string
	 */
	public function render(): string
	{
		$cell = $this->_cssCell();
		$css = $this->_cssCategory();

		switch( true )
		{
			case(!empty( $cell ) && !empty( $css )):
				return $cell . ' ' . $css;
			default:
				return $cell . $css;
		}
	}

}
