<?php

//declare(strict_types=1);

namespace Netzmacher\Tile\ViewHelpers\Get\Modul;

use Netzmacher\Tile\Domain\Repository\CategoryRepository;
use Netzmacher\Tile\Domain\Repository\CellRepository;
use Netzmacher\Tile\Domain\Repository\CssRepository;
use Netzmacher\Tile\Domain\Repository\TicketRepository;
use Netzmacher\Tile\Utility\ObjectUtility;
use TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper;

/**
 * Class CssClassViewHelper
 * 
 * @version 0.0.6
 * @since 0.0.6
 * @internal #t3816
 */
class CssClassViewHelper extends AbstractViewHelper
{

	/**
	 * @return mixed
	 */
	private function _getCategoryCss()
	{
		$default = $this->arguments[ 'default' ];
		$uid = $this->arguments[ 'uidCategory' ];
		if( empty( $uid ) )
		{
			return $default;
		}

		$categoryRepository = ObjectUtility::getObjectManager()->get( CategoryRepository::class );
		$category = $categoryRepository->findByUid( $uid );

		if( empty( $category ) )
		{
			return $default;
		}

		$categoryCss = $category->getTxTileCss();
		$categoryCssUid = $categoryCss->getUid();

		$cssRepository = ObjectUtility::getObjectManager()->get( CssRepository::class );
		$css = $cssRepository->findByUid( $categoryCssUid );

		return $css->getClass();
	}

	/**
	 * @return mixed
	 */
	private function _getGroupCss()
	{
		$default = $this->arguments[ 'default' ];
		$class = $this->arguments[ 'class' ];
		$uidCell = $this->arguments[ 'uidCell' ];

		switch( true )
		{
			case( empty( $uidCell ) ):
			case( empty( $class ) ):
				return $default;
			default :
				// follow the workflow
				break;
		}

		$cssCell = ObjectUtility::getObjectManager()->get( CellRepository::class );
		$cell = $cssCell->findByUid( $uidCell );

		if( empty( $cell ) )
		{
			return '';
		}

		return $cell->getGroupcss();
	}

	/**
	 * @param string $status: caddy | default | selled
	 * @return mixed
	 */
	private function _getTicketCss( $status = 'default' )
	{
		$default = $this->arguments[ 'default' ];
		$class = $this->arguments[ 'class' ];
		$uid = $this->arguments[ 'uidTicket' ];

		switch( true )
		{
			case( empty( $uid ) ):
			case( empty( $class ) ):
				return $default;
			default :
				// follow the workflow
				break;
		}

		$ticketRepository = ObjectUtility::getObjectManager()->get( TicketRepository::class );
		$ticket = $ticketRepository->findByUid( $uid );

		if( empty( $ticket ) )
		{
			return null;
		}

		switch( $status )
		{
			case('caddy'):
				$ticketCss = $ticket->getTxTileCsscaddy();
				break;
			case('default'):
				$ticketCss = $ticket->getTxTileCssdefault();
				break;
			case('selled'):
				$ticketCss = $ticket->getTxTileCssselled();
				break;
			default:
				$prompt = 'Undefined value in switch @ ' . __METHOD__ . ' (#' . __LINE__ . ')';
				die( $prompt );
		}

		if( empty( $ticketCss ) )
		{
			$this->_getTicketCssPromptDie( __METHOD__, __LINE__, $ticket );
		}
		$ticketCssUid = $ticketCss->getUid();

		$cssRepository = ObjectUtility::getObjectManager()->get( CssRepository::class );
		$css = $cssRepository->findByUid( $ticketCssUid );
		return $css->getClass();
	}

	/**
	 * @param string $method
	 * @param string $line
	 * @param string $ticket
	 * @return void
	 */
	private function _getTicketCssPromptDie( $method, $line, $ticket )
	{
		$prompt = 'ERROR @ ' . $method . ' (#' . $line . ')<br />'
						. '<br />'
						. 'It seems, that a CSS class of the ticket with the uid #' . $ticket->getUid() . ' is empty.<br />'
						. 'Please check the ticket!'
		;
		die( $prompt );
	}

	/**
	 * @param int $uid
	 * @return mixed
	 */
	private function _getTicketCssSelled()
	{
		$default = $this->arguments[ 'default' ];
		$class = $this->arguments[ 'class' ];
		$uid = $this->arguments[ 'uidTicket' ];

		switch( true )
		{
			case( empty( $uid ) ):
			case( empty( $class ) ):
				return $default;
			default :
				// follow the workflow
				break;
		}

		$ticketRepository = ObjectUtility::getObjectManager()->get( TicketRepository::class );
		$ticket = $ticketRepository->findByUid( $uid );

		if( empty( $ticket ) )
		{
			return null;
		}

		$ticketCss = $ticket->getTxTileCss();
		$ticketCssUid = $ticketCss->getUid();

		$cssRepository = ObjectUtility::getObjectManager()->get( CssRepository::class );
		$css = $cssRepository->findByUid( $ticketCssUid );
		return $css->getClass();
	}

	/**
	 * @return void
	 */
	public function initializeArguments()
	{
		parent::initializeArguments();
		$this->registerArgument( 'default', 'string', 'default', false, '' );
		$this->registerArgument( 'class', 'string', 'class', false, '' );
		$this->registerArgument( 'uidCategory', 'int', 'uidCategory', false, 0 );
		$this->registerArgument( 'uidCell', 'int', 'uidCell', false, 0 );
		$this->registerArgument( 'uidGroup', 'int', 'uidGroup', false, 0 );
		$this->registerArgument( 'uidTicket', 'int', 'uidTicket', false, 0 );
	}

	/**
	 * @return mixed
	 */
	public function render()
	{
		switch( $this->arguments[ 'class' ] )
		{
			case('cssClassForCategory'):
				return $this->_getCategoryCss();
			case('cssClassForGroup'):
				return $this->_getGroupCss();
			case('cssClassForTicketCaddy'):
				return $this->_getTicketCss( 'caddy' );
			case('cssClassForTicket'):
			case('cssClassForTicketDefault'):
				return $this->_getTicketCss();
			case('cssClassForTicketSelled'):
				return $this->_getTicketCss( 'selled' );
			case(''):
				return $this->arguments[ 'default' ];
			default:
				var_dump( __METHOD__, __LINE__, 'Error in switch! Unexpected value.' );
				die();
		}
	}

}
